// wzAIEngine.h - AI engine
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_AI_ENGINE_H__
#define __WZ_AI_ENGINE_H__

#include "wzMove.h"
#include <vector>

class wzGame;
class wzBoardBase;
class wzAIBoard;
class wzEvaluator;


/////////////////////
//  AI engine
///////////////////////////////////////////////////////
class wzAIEngine
{
public:
    wzAIEngine( short Level );
    virtual ~wzAIEngine();

    void  BestMove( wzGame* pGame, wzBoardBase* pBoard, wzMove& mBest );

private:
    // move: coordinates and score
    typedef struct 
    {
        wzMove     Move;
        long long  Score;
    } wzAIMove;

    // moves and iterator
    typedef std::vector<wzAIMove>  wzAIMoves;
    typedef wzAIMoves::iterator    wzAIMovesIterator;

    // compare two moves, in order to make moves in descending order
    class wzDescendMoves
    {
    public:
        bool  operator()( const wzAIMove& m1, const wzAIMove& m2 )
        {
            return( m1.Score > m2.Score );
        }
    };

    // compare two moves, in order to make moves in ascending order
    class wzAscendMoves
    {
    public:
        bool  operator()( const wzAIMove& m1, const wzAIMove& m2 )
        {
            return( m1.Score < m2.Score );
        }
    };

private:
    bool       IsLeaf( short depth, wzAIMove& move );
    wzAIMoves  GenMoves( wzSTONE stone );
    long long  AlphaBeta( short depth, long long alpha, long long beta, 
                          wzSTONE stone, bool bMax, wzAIMove& move, wzMove& mBest );

private:
    wzGame*     m_pGame;
    wzAIBoard*  m_pBoard;
    wzSTONE     m_MyColor;
    short       m_Depth;

    static wzSTONE  Toggle( wzSTONE stone ) { return (stone == wzBLACK)? wzWHITE : wzBLACK; }
    static wzEvaluator  m_Evaluator;

    // infinite
    const static long long  wzInfinite = 1000000000000000000LL;
};

#endif

