// wzBitBoard.h - Board represented by bits
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_BIT_BOARD_H__
#define __WZ_BIT_BOARD_H__

#include "wzDefs.h"


////////////////////
// game board by bits
//////////////////////////////////
class wzBitBoard
{
public:
    wzBitBoard() { Clean(); }
    wzBitBoard( const wzBitBoard& other );

    virtual ~wzBitBoard() {}

    // operators
    wzBitBoard& operator= ( const wzBitBoard& other );
    bool        operator==( const wzBitBoard& other ) const;
    bool        operator!=( const wzBitBoard& other ) const;
    short       operator& ( const wzBitBoard& other ) const;
    void        operator+=( const wzBitBoard& other );

    // inquiries
    bool  IsEmpty( const short& x, const short& y ) const;

    // operations
    bool  Set  ( const short& x, const short& y );
    void  Reset( const short& x, const short& y );
    void  Clean();

private:
    // two bytes represent a horizontal line on the board,
    // so 15x15 board only
    wzUInt8  m_Bits[2][wzDimension];

private:
    // count how my bits in a byte having been set.
    // use a pre-set array to improve performance.
    class wzBitCounter 
    {
    public:
        wzBitCounter();
        virtual ~wzBitCounter() {}

        short  operator()( const wzUInt8& byte ) const { return m_Count[byte]; }

    private:
        short  Count( const wzUInt8& Byte );
        short  m_Count[256]; 
    };

    static wzBitCounter   m_BitCounter;
    static const wzUInt8  m_OneBit[8];
    static const wzUInt8  m_7Bits[8];

private:
    static short  ByteIndex( const short& x );
    static short  BitIndex ( const short& x );
};


/////////////
//  x/8
//////////////////////////////////////////////
inline short wzBitBoard::ByteIndex( const short& x )
{
    return( (x < 8)? 0 : 1 ); 
}

///////////////
//  x%8
//////////////////////////////////////////////
inline short wzBitBoard::BitIndex( const short& x )
{
    return( (x < 8)? x : x-8 ); 
}

#endif
