// wzBoard.cpp - Game board
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzBoard.h"
#include "wzBoardView.h"
#include "wzMoveObserver.h"
#include "wzHintHandler.h"


////////////////////////////////
wzBoard::wzBoard()
  : wzBoardBase(), 
    m_pBoardView   ( new wzBoardView() ),
    m_pMoveObserver( new wzMoveObserver() ),
    m_pHintHandler ( new wzHintHandler() ),
    m_HasWinner    ( false )
{
    EnableView();
    Register( m_pMoveObserver );
    Register( m_pHintHandler );
}

////////////////////////////////
// Don't delete m_pBoardView here.
// The subclass of wzGame takes care of board view. 
///////////////////////////////////////////////////
wzBoard::~wzBoard()
{
    delete m_pMoveObserver;
    delete m_pHintHandler;
}

////////////////////////////////////////////
void wzBoard::SetView( wzBoardView* pBoardView )
{
    DisableView();
    delete m_pBoardView;

    m_pBoardView = pBoardView;
    EnableView();
}

////////////////////////////////
void wzBoard::UpdateView()
{
    m_pBoardView->UpdateView();
}

////////////////////////////////
void wzBoard::EnableView()
{
    Register( m_pBoardView );
}

////////////////////////////////
void wzBoard::DisableView()
{
    Remove( m_pBoardView );
}

////////////////////////////////////////
bool wzBoard::WaitForInput( short& x, short& y)
{
    return m_pBoardView->WaitForInput( x, y );
}

////////////////////////////////////////
void wzBoard::NotifyInput()
{
    m_pBoardView->NotifyInput();
}

////////////////////////////////
void wzBoard::SetWinningRow( const wzBound& row )
{
    m_HasWinner  = true;
    m_WinningRow = row;
}

////////////////////////////////
bool wzBoard::HasHint() const
{
    return m_pHintHandler->HasHint();
}

////////////////////////////////
wzMove wzBoard::GetHint() const
{
    return m_pHintHandler->GetHint();
}

////////////////////////////////
void wzBoard::SetHint( const wzMove& hint )
{
    m_pHintHandler->SetHint( hint );
    UpdateView();
}

////////////////////////////////
void wzBoard::RemoveHint()
{
    m_pHintHandler->Reset();
    UpdateView();
}

////////////////////////////////
void wzBoard::Clean()
{
    m_HasWinner = false;
    wzBoardBase::Clean();
}

////////////////////////////////
bool wzBoard::UndoMove()
{
    if( IsEmpty() )
        return false;

    m_HasWinner = false;

    wzMove move = GetLastMove();
    Reset( move );

    return true;
}

///////////////////////////////////////
void wzBoard::CopyMoves( wzMoves& moves ) const
{
    m_pMoveObserver->CopyMoves( moves );
}

///////////////////////////////////////
wzMove wzBoard::GetLastMove() const 
{ 
    return m_pMoveObserver->GetLastMove(); 
}
