// wzBoard.h - Game board
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_BOARD_H__
#define __WZ_BOARD_H__

#include "wzBoardBase.h"
#include "wzMove.h"
#include "wzBound.h"

class wzBoardView;
class wzMoveObserver;
class wzHintHandler;


////////////////////
// Game board. Its view takes care of the visualization  
//////////////////////////////////////////////////////////
class wzBoard : public wzBoardBase
{
public:
    wzBoard();
    virtual ~wzBoard();

    // board view
    void  SetView( wzBoardView* pBoardView );
    void  UpdateView();
    void  EnableView();
    void  DisableView();

    // interact with human player
    bool  WaitForInput( short& x, short& y);
    void  NotifyInput();

    // operations
    void  Clean();
    bool  UndoMove();

    // moves
    void    CopyMoves( wzMoves& moves ) const;
    wzMove  GetLastMove() const;

    // winning row
    bool     HasWinner() const  { return m_HasWinner; }
    void     SetWinningRow( const wzBound& row );
    wzBound  GetWinningRow() const { return m_WinningRow; }

    // hint
    bool    HasHint() const;
    wzMove  GetHint() const;
    void    SetHint( const wzMove& hint );
    void    RemoveHint();

private:
    wzBoardView*     m_pBoardView;      // board view, one view only
    wzMoveObserver*  m_pMoveObserver;   // moves
    wzHintHandler*   m_pHintHandler;    // hint

    // winning row
    bool     m_HasWinner;
    wzBound  m_WinningRow;

private:
    wzBoard( const wzBoard& );
    wzBoard& operator=( const wzBoard& );
};

#endif
