// wzBoundObserver.cpp - Track changes of the bounding box of occupied fields.
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzBoundObserver.h"
#include "wzBoardBase.h"


////////////////////////////////////////////
void wzBoundObserver::Init( wzBoardBase* pBoardBase )
{
    Clean();
    for( short y = 0; y < wzDimension; ++y )
    {
        for( short x = 0; x < wzDimension; ++x )
        {
            if( !pBoardBase->IsEmpty( x, y ) )
            {
                if( x < m_Bound.xMin )  m_Bound.xMin = x;
                if( y < m_Bound.yMin )  m_Bound.yMin = y;
                if( x > m_Bound.xMax )  m_Bound.xMax = x;
                if( y > m_Bound.yMax )  m_Bound.yMax = y;
            }
        }
    }

    // save the initial bounding box
    m_Stack.push( m_Bound );
}

/////////////////////////////////////
void wzBoundObserver::Set( const wzMove& m )
{
    // update bounding box
    if( m.X < m_Bound.xMin )  m_Bound.xMin = m.X;
    if( m.Y < m_Bound.yMin )  m_Bound.yMin = m.Y;
    if( m.X > m_Bound.xMax )  m_Bound.xMax = m.X;
    if( m.Y > m_Bound.yMax )  m_Bound.yMax = m.Y;

    m_Stack.push( m_Bound );
}

///////////////////
// FIFO assumption applies here
////////////////////////////////////////////
void wzBoundObserver::Reset( const wzMove& m )
{
    // Guarded by wzBoardBase::Reset(x,y)
    // no need to check if it's empty.
    m_Stack.pop();
    m_Bound = m_Stack.top();
}

////////////////////////////////////////////
void wzBoundObserver::Clean()
{
    m_Bound.xMin = wzDimension;
    m_Bound.yMin = wzDimension;
    m_Bound.xMax = 0;
    m_Bound.yMax = 0;

    while( !m_Stack.empty() )  m_Stack.pop();
}
