// wzBoardObserver.h - Track changes of the bounding box of occupied fields.
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_BOUND_OBSERVER_H__
#define __WZ_BOUND_OBSERVER_H__

#include "wzBoardObserver.h"
#include "wzBound.h"
#include <stack>


///////////////////
// Bound observer: 
//   Track changes of the bounding box of occupied fields
// 
//   Assumptions: Stones are put on board in FIFO. 
//                (Alphe-beta search is doing in this way.)
//////////////////////////////////////////////////////////////
class wzBoundObserver : public wzBoardObserver
{
public:
    wzBoundObserver()  {}
    virtual ~wzBoundObserver() {}

    wzBound GetBound() const  { return m_Bound; }

    // override
    virtual void  Init ( wzBoardBase* pBoardBase );
    virtual void  Set  ( const wzMove& m );
    virtual void  Reset( const wzMove& m );
    virtual void  Clean();

private:
    // current bounding box:
    // smallest retangle that holds all occupied fields
    wzBound  m_Bound; 

    // stack to track bounds
    typedef std::stack<wzBound>  wzBoundStack;
    wzBoundStack  m_Stack;

private:
    wzBoundObserver( const wzBoundObserver& );
    wzBoundObserver& operator=( const wzBoundObserver& );
};

#endif
