// wzClock.h - Clock to count time spent by players
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_CLOCK_H__
#define __WZ_CLOCK_H__

#include "wzDefs.h"

#include <string>
#include <iosfwd>
#include <sys/time.h>


///////////////////////////////////////////////////////////
class wzClock
{
    friend class wzClockTest;

public:
    wzClock()  { Reset(); }
    ~wzClock() {}

    bool  IsRunning() const;
    void  Reset();
    bool  Start();
    bool  Stop();
	
    std::string  GetTime();

    friend std::ifstream&  operator>>( std::ifstream&, wzClock& );
    friend std::ofstream&  operator<<( std::ofstream&, wzClock& );

private:
    bool  ExceedLimit() const;
    void  Update();
    bool  Convert( int& hours, int& minutes, int& seconds, int& decisecs );

    static int  Centi2Deci( const long& centisecs );

private:
    timeval  m_Start, m_Stop;
    long     m_Elapsed;          // in centi-seconds
    bool     m_Running;

    static const long  m_Limit;  // in centi-seconds
};


/////////////////////////////////////
inline bool wzClock::IsRunning() const  
{ 
    return m_Running; 
}

///////////////////////////////////////
inline bool wzClock::ExceedLimit() const
{
    return( m_Elapsed >= m_Limit );
}

//////////////
// round up centi-secs to deci-secs
///////////////////////////////////////
inline int wzClock::Centi2Deci( const long& centisecs )
{
    return static_cast<int>( (centisecs + 5)/10 );
}

#define MAX_HOURS    99
#define MAX_MINUTES  59
#define MAX_SECONDS  59
#define MAX_DECISECS  9

#endif
