// wzEvaluator.cpp - Evaluate of a board position
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzEvaluator.h"
#include "wzAIBoard.h"

// initialize static data member
const long long wzEvaluator::m_Weights[m_Threats] = 
{
    1LL, 1000LL, 1000000LL, 1000000000LL, 1000000000000LL
};


///////////////////////////////////
void wzEvaluator::Reset()
{
    for( int i = 0; i < m_Threats; ++i )
    {
        m_BThreats[i] = 0;
        m_WThreats[i] = 0;
    }
}

///////////////////////////////////
// Evaluate a board postion
////////////////////////////////////////////////////////////
long long wzEvaluator::Evaluate( const wzAIBoard& b, wzSTONE myColor )
{
    // reset those counters
    Reset();

    wzBound box = b.GetBound();

    short margin = 2;
    box.xMin -= margin;
    if( box.xMin < 0 )  box.xMin = 0;

    box.yMin -= margin;
    if( box.yMin < 0 )  box.yMin = 0;

    box.xMax += margin;
    //if( box.xMax - box.xMin < 4 )  box.xMax = box.xMin + 4;
    if( box.xMax >= wzDimension ) box.xMax = wzDimension-1;

    box.yMax += margin;
    //if( box.yMax - box.yMin < 4 )  box.yMax = box.yMin + 4;
    if( box.yMax >= wzDimension ) box.yMax = wzDimension-1;

    wzPatternIterator& ith5 = m_Horizontal5.GetIterator();
    if( ith5.SetBound( box ) )  Count( b, ith5 );

    wzPatternIterator& itv5 = m_Vertical5.GetIterator();
    if( itv5.SetBound( box ) )  Count( b, itv5 );

    wzPatternIterator& itd5f = m_Diagonal5F.GetIterator();
    if( itd5f.SetBound( box ) )  Count( b, itd5f );

    wzPatternIterator& itd5b = m_Diagonal5B.GetIterator();
    if( itd5b.SetBound( box ) )  Count( b, itd5b );

    // score of black stones
    long long  score = 0LL;
    for( int i = 0; i < m_Threats; ++i )
        score += (m_BThreats[i] - m_WThreats[i])*m_Weights[i];

    return (myColor == wzBLACK)? score: -score;
}

///////////////////////////////////
// Evaluate a board postion with last move
////////////////////////////////////////////////////////////
long long wzEvaluator::Evaluate( const wzAIBoard& b, wzSTONE myColor, short x, short y )
{
    // reset those counters
    Reset();

    wzPatternIterator& ith5 = m_Horizontal5.GetIterator();
    if( ith5.SetCenter( x, y ) )  Count( b, ith5 );

    wzPatternIterator& itv5 = m_Vertical5.GetIterator();
    if( itv5.SetCenter( x, y ) )  Count( b, itv5 );

    wzPatternIterator& itd5f = m_Diagonal5F.GetIterator();
    if( itd5f.SetCenter( x, y ) )  Count( b, itd5f );

    wzPatternIterator& itd5b = m_Diagonal5B.GetIterator();
    if( itd5b.SetCenter( x, y ) )  Count( b, itd5b );

    // score of black stones
    long long  score = 0LL;
    for( int i = 0; i < m_Threats; ++i )
        score += (m_BThreats[i] - m_WThreats[i])*m_Weights[i];

    return (myColor == wzBLACK)? score: -score;
}

///////////////////////////////////
// Evaluate a board postion with last move
////////////////////////////////////////////////////////////
long long wzEvaluator::Evaluate( const wzAIBoard& b, wzSTONE myColor, const wzMove& move )
{
    return Evaluate( b, myColor, move.X, move.Y );
}

///////////////////////////////////
// actual counting for one pattern
/////////////////////////////////////////////////////////////////
void wzEvaluator::Count( const wzBoardBase& b, wzPatternIterator& it )
{
    short  nBlack = 0;
    short  nWhite = 0;

    do
    {
        nBlack = b.m_BlackStones & it.GetPattern();
        nWhite = b.m_WhiteStones & it.GetPattern();

        if( nBlack != 0 && nWhite == 0 )
            m_BThreats[nBlack-1] += 1;
        else if( nBlack == 0 && nWhite != 0 )
            m_WThreats[nWhite-1] += 1;

    } while( it.Next() );
}
