// wzEvaluator.h - Evaluate of a board position
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_EVALUATOR_H__
#define __WZ_EVALUATOR_H__

#include "wzPatternIterator.h"
#include "wzMove.h"

class wzAIBoard;
class wzBoardBase;


///////////////////////
// identify winner and evaluate scores of board
///////////////////////////////////////////////////////////////////
class wzEvaluator
{
public:
    wzEvaluator() {} 
    virtual ~wzEvaluator() {} 

    long long  Evaluate( const wzAIBoard& b, wzSTONE myColor );
    long long  Evaluate( const wzAIBoard& b, wzSTONE myColor, short x, short y );
    long long  Evaluate( const wzAIBoard& b, wzSTONE myColor, const wzMove& move );

private:
    void  Reset();
    void  Count( const wzBoardBase& b, wzPatternIterator& it );

private:
    static const short      m_Threats = 5;  // # of threat kinds
    static const long long  m_Weights[m_Threats];

    short  m_BThreats[m_Threats]; // black threats 
    short  m_WThreats[m_Threats]; // white threats
    // m_BThreats[0]: # of 1-in-rows
    // m_BThreats[1]: # of 2-in-rows
    // m_BThreats[2]: # of 3-in-rows
    // m_BThreats[3]: # of 4-in-rows
    // m_BThreats[4]: # of 5-in-rows
    // 1-in-row: there's one in the row, while the other four are empty
    // 2-in-row: there're two in the row, while the remaining three is empty
    // 5-in-row: all five are ocupied in the row

    // winning patterns
    wzHorizontal5  m_Horizontal5;
    wzVertical5    m_Vertical5;
    wzDiagonal5F   m_Diagonal5F;
    wzDiagonal5B   m_Diagonal5B;

private:
    wzEvaluator( const wzEvaluator& other );
    wzEvaluator& operator=( const wzEvaluator& other );
};

#endif
