// wzFieldObserver.h - Track fields that are affected by stones on the board.
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_FIELD_OBSERVER_H__
#define __WZ_FIELD_OBSERVER_H__

#include "wzBoardObserver.h"
#include "wzPattern.h"
#include <stack>


///////////////////
// Field observer: 
//   Track fields that are affected by stones on the board.
// 
//   Assumptions: Stones are put on board in FIFO. 
//////////////////////////////////////////////////////////////
class wzFieldObserver : public wzBoardObserver
{
public:
    wzFieldObserver() {}
    virtual ~wzFieldObserver() {}

    const wzBitBoard&  GetFields() const  { return m_Fields; }

    // override
    virtual void  Init ( wzBoardBase* pBoardBase );
    virtual void  Set  ( const wzMove& m );
    virtual void  Reset( const wzMove& m );
    virtual void  Clean()  { while( !m_Stack.empty() )  m_Stack.pop(); }

private:
    // current fileds
    wzBitBoard  m_Fields;

    // stack to track fields
    typedef std::stack<wzBitBoard>  wzFieldStack;
    wzFieldStack  m_Stack;

    // star patterns
    wzStarPattern  m_StarPattern;

private:
    wzFieldObserver( const wzFieldObserver& );
    wzFieldObserver& operator=( const wzFieldObserver& );
};

#endif
