// wzGameFile.h - The game file
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_GAME_FILE_H__
#define __WZ_GAME_FILE_H__

#include "wzBoardObserver.h"

#include <string>
#include <iosfwd>

class wzGame;
class wzBoard;
class wzVersion;


////////////////////////////////////////
class wzGameFile : public wzBoardObserver
{
    friend class wzGameFileTest;

public:
    wzGameFile( wzGame* pGame );
    ~wzGameFile() {}

    void  Reset();

    bool  Open();
    void  Save();
    void  SaveAs();

    // inquiries
    std::string  GetFileName() const;
    bool         IsSaved()     const;

    // overrides of wzBoardObserver
    virtual void  Init ( wzBoardBase* )     {}
    virtual void  Clean()                   {}
    virtual void  Set  ( const wzMove& m )  { BoardChanged(); }
    virtual void  Reset( const wzMove& m )  { BoardChanged(); }

private:
    bool  PromptFile( bool bForOpen );

    void  DoSave();
    void  Write( std::ofstream& );
    void  WriteTime ( std::ofstream& );
    void  WriteMoves( std::ofstream& );

    void  Read( std::ifstream& );
    void  ReadTime  ( std::ifstream& );
    void  ApplyMoves( std::ifstream& );

    void  BoardChanged(); 
    void  BeSaved(); 

private:
    wzGame*      m_pGame;
    wzBoard*     m_pBoard;

    std::string  m_FileName;
    bool         m_Saved;

private:
    static bool  ValidID( const char ID[2] );
    static bool  ValidTrailer( const char  );

private:
    static const char       m_MagicID[2];
    static const char       m_Trailer;
    static const wzVersion  m_Version;
};


////////////////////////////////////
inline bool wzGameFile::ValidID( const char ID[2] ) 
{ 
    return( ID[0] == m_MagicID[0] && ID[1] == m_MagicID[1] ); 
}

////////////////////////////////////
inline bool wzGameFile::ValidTrailer( const char Trailer ) 
{ 
    return( Trailer == m_Trailer ); 
}

#endif
