// wzGameReplayer.cpp - The game replayer class
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzGameReplayer.h"
#include "wzGame.h"
#include "wzBoard.h"
#include "wzReferee.h"


/////////////////////////
// back to the 1st ply
////////////////////////////////////////
void wzGameReplayer::FirstPly()
{
    // if 1st time 
    if( m_Moves.empty() )
        m_pBoard->CopyMoves( m_Moves );

    m_pBoard->Clean();

    m_MovesIt = m_Moves.begin();
    if( m_MovesIt != m_Moves.end() )
        m_pBoard->Set( *m_MovesIt );
}

////////////////////////////
// step back to previous ply
/////////////////////////////////////////
void wzGameReplayer::PrevPly()
{
    if( m_MovesIt > m_Moves.begin() )
    {
        m_pBoard->UndoMove();
        m_MovesIt--;
    }
    else
    {
        m_MovesIt = m_Moves.begin();
    }
}

///////////////////////////
// step forward to next ply
////////////////////////////////////////////////
void wzGameReplayer::NextPly()
{
    m_MovesIt++;
    if( m_MovesIt < m_Moves.end() )
    {
        m_pBoard->Set( *m_MovesIt );

        // mark winning row
        if( m_MovesIt == --m_Moves.end() )
            wzGame::m_Referee.Winner( *m_pBoard );
    }
    else
    {
        m_MovesIt--;
    }
}

/////////////////////
// jump to last ply
//////////////////////////////////////////////////
void wzGameReplayer::LastPly()
{
    for( ; m_MovesIt != m_Moves.end(); m_MovesIt++ )
        m_pBoard->Set( *m_MovesIt );

    // mark winning row
    wzGame::m_Referee.Winner( *m_pBoard );

    m_MovesIt--;
}
