// wzGameRunner.cpp - The game runner class
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzGameRunner.h"
#include "wzGame.h"
#include "wzBoard.h"
#include "wzPlayer.h"
#include "wzReferee.h"

////////////////////////////////////////////
wzGameRunner::wzGameRunner( wzGame* pGame )
  : m_pGame( pGame),
    m_pBoard( pGame->m_pBoard ),
    m_CurrentPlayer( 0 )
{
    m_pPlayers[0] = new wzPlayer( wzBLACK ); 
    m_pPlayers[0]->Set( m_pGame, m_pBoard );
    m_pPlayers[1] = new wzPlayer( wzWHITE ); 
    m_pPlayers[1]->Set( m_pGame, m_pBoard );
}


////////////////////////////
wzGameRunner::~wzGameRunner() 
{
    delete m_pPlayers[0];
    delete m_pPlayers[1];
}

/////////////
// set black stone player
////////////////////////////////////////////
void wzGameRunner::SetBlack( wzPlayer* pPlayer ) 
{ 
    delete m_pPlayers[0];

    m_pPlayers[0] = pPlayer; 
    m_pPlayers[0]->Set( m_pGame, m_pBoard );
}

/////////////
// set white stone player
////////////////////////////////////////////
void wzGameRunner::SetWhite( wzPlayer* pPlayer ) 
{ 
    delete m_pPlayers[1];

    m_pPlayers[1] = pPlayer; 
    m_pPlayers[1]->Set( m_pGame, m_pBoard );
}

//////////////////////////////////////////////
void wzGameRunner::Run()
{
    m_pPlayers[0]->Init();
    m_pPlayers[1]->Init();

    wzSTONE  winner = wzNONE;
    wzMove   move; 
    bool     bMadeAMove;

    // game loop
    while( m_pGame->IsRunning() )
    {
        winner = wzGame::m_Referee.Winner( *m_pBoard );
        if( winner == wzNONE && !m_pBoard->IsFull() )
        {
            m_pGame->StartMove( m_CurrentPlayer );
            bMadeAMove = m_pPlayers[m_CurrentPlayer]->MakeAMove( move );
            m_pGame->FinishMove( m_CurrentPlayer );

            if( m_pGame->KeepRunning() && bMadeAMove )
            {
                m_pBoard->Set( move );
                SwitchPlayer();
            }
            else if( m_pGame->NeedUndo() )
                Revert();
            else if( m_pGame->NeedSave() )
                DoSave();
            else if( m_pGame->NeedSaveAs() )
                DoSaveAs();
        }
        else
        {
            m_pGame->BeEndedState();

            if( winner != wzNONE ) 
                m_pBoard->UpdateView();
            
        }
    } 

    GameEnded( winner );
}

///////////////////////
// game end, send a notification
/////////////////////////////////////////////
void wzGameRunner::GameEnded( wzSTONE winner )
{
    if( winner == wzBLACK )
        m_pGame->BlackWins();
    else if( winner == wzWHITE )
        m_pGame->WhiteWins();
    else if( m_pBoard->IsFull() )
        m_pGame->GameDraw();
    else 
        m_pGame->GameStopped();
}

////////////////////////////////////
void wzGameRunner::Revert()
{
    // revert two plies at most
    short nRevert = 0;
    if( m_pBoard->UndoMove() )  nRevert++;
    if( m_pBoard->UndoMove() )  nRevert++;

    // only one stone on board
    if( nRevert == 1 )  SwitchPlayer();

    m_pGame->BeRunningState();
}

////////////////////////////////////
void wzGameRunner::DoSave()
{
    m_pGame->DoSaveGame();
    m_pGame->BeRunningState();
}

////////////////////////////////////
void wzGameRunner::DoSaveAs()
{
    m_pGame->DoSaveGameAs();
    m_pGame->BeRunningState();
}
