// wzGameState.cpp - The game states
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzGameState.h"
#include "wzGame.h"


//             | Idle     Running  Hint    Undo   Save   SaveAs  Ended      Replaying
// ------------+----------------------------------------------------------------------
//  NewGame    | Running                                         Running    Running
//  OpenGame   | Running                                         Running    Running
//  SaveGame   |          Save     Save                          Ended      
//  SaveGameAs |          SaveAS   SaveAs                        Ended      Replaying
//  StopGame   |          Ended    Ended   Ended  Ended  Ended
//  UndoMove   |          Undo     Undo
//  HintMove   |          Hint
//  FirstPly   |                                                 Replaying  Replaying
//  PrevPly    |                                                            Replaying
//  NextPly    |                                                            Replaying
//  LastPly    |                                                            Replaying
//  SetOptions | Idle                                            Idle       Idle


//////////////////////////////////
void wzGameStateImpl::NewGame( wzGame* pGame )
{
    pGame->ChangeState( wzGameRunningState::Instance() );
    pGame->DoNewGame();
}

//////////////////////////////////
void wzGameStateImpl::OpenGame( wzGame* pGame )
{
    int Ret = pGame->DoOpenGame();
    if( Ret == wzGame::FILE_OK )
    {
        pGame->ChangeState( wzGameRunningState::Instance() );
        pGame->Run();
    }
    else if( Ret == wzGame::FILE_ERROR )
    {
        pGame->ChangeState( wzGameIdleState::Instance() );
    }
}

///////////////////////////////////////////////////
bool wzGameStateImpl::SetOptions( wzGame* pGame, const wzOptions& options )
{
    if( !pGame->DoSetOptions( options ) )
        return false;

    pGame->ChangeState( wzGameIdleState::Instance() );
    pGame->Reset();

    return true;
}

//////////////////////////////////
void wzGameRunningStateImpl::SaveGame( wzGame* pGame )
{
    pGame->BeSaveState();
}

//////////////////////////////////
void wzGameRunningStateImpl::SaveGameAs( wzGame* pGame )
{
    pGame->BeSaveAsState();
}

//////////////////////////////////
void wzGameRunningStateImpl::StopGame( wzGame* pGame )
{
    pGame->BeEndedState();
}

//////////////////////////////////
void wzGameRunningStateImpl::UndoMove( wzGame* pGame )
{
    pGame->BeUndoState();
}

//////////////////////////////////
void wzGameRunningSubStateImpl::StopGame( wzGame* pGame )
{
    pGame->BeEndedState();
}

////////////////
// static data members of wzGameIdleState
//////////////////////////////////////////////////
wzGameState*            wzGameIdleState::m_pIdleState = NULL;
wzGameState::Destoryer  wzGameIdleState::m_Destoryer;

///////////////////////////
wzGameState* wzGameIdleState::Instance()
{
    if( m_pIdleState == NULL )
    {
        m_pIdleState = new wzGameIdleState();
        m_Destoryer.Set( m_pIdleState );
    }

    return m_pIdleState;
}

////////////////
// static data members of wzGameRunningState
//////////////////////////////////////////////////
wzGameState*            wzGameRunningState::m_pRunningState = NULL;
wzGameState::Destoryer  wzGameRunningState::m_Destoryer;

/////////////////////////////////////////
wzGameState* wzGameRunningState::Instance()
{
    if( m_pRunningState == NULL )
    {
        m_pRunningState = new wzGameRunningState();
        m_Destoryer.Set( m_pRunningState );
    }

    return m_pRunningState;
}

//////////////////////////////////
void wzGameRunningState::HintMove( wzGame* pGame )
{
    pGame->BeHintState();
}

////////////////
// static data members of wzGameHintState
//////////////////////////////////////////////////
wzGameState*            wzGameHintState::m_pHintState = NULL;
wzGameState::Destoryer  wzGameHintState::m_Destoryer;

//////////////////////////////////////
wzGameState* wzGameHintState::Instance()
{
    if( m_pHintState == NULL )
    {
        m_pHintState = new wzGameHintState();
        m_Destoryer.Set( m_pHintState );
    }

    return m_pHintState;
}

////////////////
// static data members of wzGameUndoState
//////////////////////////////////////////////////
wzGameState*            wzGameUndoState::m_pUndoState = NULL;
wzGameState::Destoryer  wzGameUndoState::m_Destoryer;

//////////////////////////////////////
wzGameState* wzGameUndoState::Instance()
{
    if( m_pUndoState == NULL )
    {
        m_pUndoState = new wzGameUndoState();
        m_Destoryer.Set( m_pUndoState );
    }

    return m_pUndoState;
}

////////////////
// static data members of wzGameSaveState
//////////////////////////////////////////////////
wzGameState*            wzGameSaveState::m_pSaveState = NULL;
wzGameState::Destoryer  wzGameSaveState::m_Destoryer;

//////////////////////////////////////
wzGameState* wzGameSaveState::Instance()
{
    if( m_pSaveState == NULL )
    {
        m_pSaveState = new wzGameSaveState();
        m_Destoryer.Set( m_pSaveState );
    }

    return m_pSaveState;
}

////////////////
// static data members of wzGameSaveAsState
//////////////////////////////////////////////////
wzGameState*            wzGameSaveAsState::m_pSaveAsState = NULL;
wzGameState::Destoryer  wzGameSaveAsState::m_Destoryer;

//////////////////////////////////////
wzGameState* wzGameSaveAsState::Instance()
{
    if( m_pSaveAsState == NULL )
    {
        m_pSaveAsState = new wzGameSaveAsState();
        m_Destoryer.Set( m_pSaveAsState );
    }

    return m_pSaveAsState;
}

////////////////
// static data members of wzGameEndedState
//////////////////////////////////////////////////
wzGameState*            wzGameEndedState::m_pEndedState = NULL;
wzGameState::Destoryer  wzGameEndedState::m_Destoryer;

///////////////////////////////////
wzGameState* wzGameEndedState::Instance()
{
    if( m_pEndedState == NULL )
    {
        m_pEndedState = new wzGameEndedState();
        m_Destoryer.Set( m_pEndedState );
    }

    return m_pEndedState;
}

//////////////////////////////////
void wzGameEndedState::SaveGame( wzGame* pGame )
{
    pGame->DoSaveGame();
}

//////////////////////////////////
void wzGameEndedState::SaveGameAs( wzGame* pGame )
{
    pGame->DoSaveGameAs();
}

//////////////////////////////////
void wzGameEndedState::FirstPly( wzGame* pGame )
{
    pGame->ChangeState( wzGameReplayingState::Instance() );
    pGame->DoFirstPly();
}

////////////////
// static data members of wzGameReplayingState
//////////////////////////////////////////////////
wzGameState*            wzGameReplayingState::m_pReplayingState = NULL;
wzGameState::Destoryer  wzGameReplayingState::m_Destoryer;

/////////////////////////////////////
wzGameState* wzGameReplayingState::Instance()
{
    if( m_pReplayingState == NULL )
    {
        m_pReplayingState = new wzGameReplayingState();
        m_Destoryer.Set( m_pReplayingState );
    }

    return m_pReplayingState;
}

//////////////////////////////////
void wzGameReplayingState::SaveGameAs( wzGame* pGame )
{
    pGame->DoSaveGameAs();
}

//////////////////////////////////
void wzGameReplayingState::FirstPly( wzGame* pGame )
{
    pGame->DoFirstPly();
}

//////////////////////////////////
void wzGameReplayingState::PrevPly( wzGame* pGame )
{
    pGame->DoPrevPly();
}

//////////////////////////////////
void wzGameReplayingState::NextPly( wzGame* pGame )
{
    pGame->DoNextPly();
}

//////////////////////////////////
void wzGameReplayingState::LastPly( wzGame* pGame )
{
    pGame->DoLastPly();
}
