// wzGameState.h - The game states
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_GAME_STATE_H__
#define __WZ_GAME_STATE_H__

#ifndef NULL
#define NULL  0
#endif

class wzGame;
class wzOptions;


//////////////////////////////
// Game state base
// Default impl that does nothing
///////////////////////////////////////////////////////////
class wzGameState
{
public:
    virtual void  NewGame ( wzGame* pGame )   {}
    virtual void  OpenGame( wzGame* pGame )   {}
    virtual bool  SetOptions( wzGame* pGame, const wzOptions& options ) { return false; }
    virtual void  SaveGame( wzGame* pGame )   {}
    virtual void  SaveGameAs( wzGame* pGame ) {}
    virtual void  StopGame( wzGame* pGame )   {}
    virtual void  UndoMove( wzGame* pGame )   {}
    virtual void  HintMove( wzGame* pGame )   {}
    virtual void  FirstPly( wzGame* pGame )   {}
    virtual void  PrevPly ( wzGame* pGame )   {}
    virtual void  NextPly ( wzGame* pGame )   {}
    virtual void  LastPly ( wzGame* pGame )   {}

protected:
    wzGameState() {}
    virtual ~wzGameState() {}

    // singleton destoryer
    class Destoryer
    {
    public:
        Destoryer() : m_pState( NULL ) {}
        ~Destoryer() { delete m_pState; }

        void Set( wzGameState* pState ) { m_pState = pState; }

    private:
        // object to be destoryed
        wzGameState*  m_pState;

        // avoid double deletion
        Destoryer( const Destoryer& );
        Destoryer& operator=( const Destoryer& );
    };

private:
    wzGameState( const wzGameState& );
    wzGameState& operator=( const wzGameState& );
};


///////////////////////////////
// Default impl of Running state
///////////////////////////////////////////////////
class wzGameRunningStateImpl : public wzGameState
{
public:
    virtual void  SaveGame( wzGame* pGame );
    virtual void  SaveGameAs( wzGame* pGame );
    virtual void  StopGame( wzGame* pGame );
    virtual void  UndoMove( wzGame* pGame );

protected:
    wzGameRunningStateImpl() : wzGameState() {}
    virtual ~wzGameRunningStateImpl() {}
};


////////////////////////////////////////////////////////
class wzGameRunningState : public wzGameRunningStateImpl
{
public:
    static wzGameState*  Instance();

    virtual void  HintMove( wzGame* pGame );

protected:
    wzGameRunningState() : wzGameRunningStateImpl() {}
    virtual ~wzGameRunningState() {}

private:
    static wzGameState*            m_pRunningState;
    static wzGameState::Destoryer  m_Destoryer;
};


//////////////////////////////////////////////////
class wzGameHintState : public wzGameRunningStateImpl
{
public:
    static wzGameState*  Instance();

protected:
    wzGameHintState() : wzGameRunningStateImpl() {}
    virtual ~wzGameHintState() {}

private:
    static wzGameState*            m_pHintState;
    static wzGameState::Destoryer  m_Destoryer;
};


////////////////////////////////////
// Default impl of Running substate
/////////////////////////////////////////////////
class wzGameRunningSubStateImpl : public wzGameState
{
public:
    virtual void  StopGame( wzGame* pGame );

protected:
    wzGameRunningSubStateImpl() : wzGameState() {}
    virtual ~wzGameRunningSubStateImpl() {}
};


///////////////////////////////////////////////////////
class wzGameUndoState : public wzGameRunningSubStateImpl
{
public:
    static wzGameState*  Instance();

protected:
    wzGameUndoState() : wzGameRunningSubStateImpl() {}
    virtual ~wzGameUndoState() {}

private:
    static wzGameState*            m_pUndoState;
    static wzGameState::Destoryer  m_Destoryer;
};


//////////////////////////////////////////////////////
class wzGameSaveState : public wzGameRunningSubStateImpl
{
public:
    static wzGameState*  Instance();

protected:
    wzGameSaveState() : wzGameRunningSubStateImpl() {}
    virtual ~wzGameSaveState() {}

private:
    static wzGameState*            m_pSaveState;
    static wzGameState::Destoryer  m_Destoryer;
};


///////////////////////////////////////////////////////////
class wzGameSaveAsState : public wzGameRunningSubStateImpl
{
public:
    static wzGameState*  Instance();

protected:
    wzGameSaveAsState() : wzGameRunningSubStateImpl() {}
    virtual ~wzGameSaveAsState() {}

private:
    static wzGameState*            m_pSaveAsState;
    static wzGameState::Destoryer  m_Destoryer;
};


////////////////
// Another impl 
//////////////////////////////////////////
class wzGameStateImpl : public wzGameState
{
public:
    static wzGameState*  Instance();

    virtual void  NewGame ( wzGame* pGame );
    virtual void  OpenGame( wzGame* pGame );
    virtual bool  SetOptions( wzGame* pGame, const wzOptions& options );

protected:
    wzGameStateImpl() : wzGameState() {}
    virtual ~wzGameStateImpl() {}
};


///////////////////////////////////////////////
class wzGameIdleState : public wzGameStateImpl
{
public:
    static wzGameState*  Instance();

protected:
    wzGameIdleState() : wzGameStateImpl() {}
    virtual ~wzGameIdleState() {}

private:
    static wzGameState*            m_pIdleState;
    static wzGameState::Destoryer  m_Destoryer;
};


///////////////////////////////////////////////
class wzGameEndedState : public wzGameStateImpl
{
public:
    static wzGameState*  Instance();

    virtual void  SaveGame( wzGame* pGame );
    virtual void  SaveGameAs( wzGame* pGame );
    virtual void  FirstPly( wzGame* pGame );

protected:
    wzGameEndedState() : wzGameStateImpl() {}
    virtual ~wzGameEndedState() {}

private:
    static wzGameState*            m_pEndedState;
    static wzGameState::Destoryer  m_Destoryer;
};


///////////////////////////////////////////////
class wzGameReplayingState : public wzGameStateImpl
{
public:
    static wzGameState*  Instance();

    virtual void  SaveGameAs( wzGame* pGame );
    virtual void  FirstPly( wzGame* pGame );
    virtual void  PrevPly ( wzGame* pGame );
    virtual void  NextPly ( wzGame* pGame );
    virtual void  LastPly ( wzGame* pGame );

protected:
    wzGameReplayingState() : wzGameStateImpl() {}
    virtual ~wzGameReplayingState() {}

private:
    static wzGameState*            m_pReplayingState;
    static wzGameState::Destoryer  m_Destoryer;
};

#endif
