// wzHintHandler.h - Help wzBoard handling hint.
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_HINT_HANDLER_H__
#define __WZ_HINT_HANDLER_H__

#include "wzBoardObserver.h"


///////////////////
// Help wzBoard handling hint move. It watches the board changes.
//
// Flag m_Hint indicates that there's a hint on the board and the board view can
// visualize it, while flag wzGame::m_NeedHint directs wzHumanPlayer to call 
// wzAIEngine to get a hint, so they do not need to be synchronized.
/////////////////////////////////////////////////////////////////////////////////
class wzHintHandler : public wzBoardObserver
{
public:
    wzHintHandler() {}
    virtual ~wzHintHandler() {}

    bool    HasHint() const  { return m_HasHint; }
    wzMove  GetHint() const  { return m_Hint; }
    void    SetHint( const wzMove& hint )  { m_Hint = hint; m_HasHint = true; }
    void    Reset()  { m_HasHint = false; }

    // overrides: whenever board changes, reset the flag.
    virtual void  Init ( wzBoardBase* pBoardBase )  { Reset(); }
    virtual void  Set  ( const wzMove& m )          { Reset(); }
    virtual void  Reset( const wzMove& m )          { Reset(); }
    virtual void  Clean()                           { Reset(); }

private:
    bool    m_HasHint;
    wzMove  m_Hint;

private:
    wzHintHandler( const wzHintHandler& );
    wzHintHandler& operator=( const wzHintHandler& );
};

#endif
