// wzHumanPlayer.cpp - Human player, subclass of wzPlayer
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzHumanPlayer.h"
#include "wzBoard.h"
#include "wzGame.h"
#include "wzAIEngine.h"


/////////////////////////////////////////////////////////////////////
wzHumanPlayer::wzHumanPlayer( wzSTONE stone ) 
  : wzPlayer( stone ),
    m_pAIEngine( new wzAIEngine( 1 ) )
{
}

//////////////////////////////
wzHumanPlayer::~wzHumanPlayer() 
{
    delete m_pAIEngine;
}

/////////////////////////////////////////////////////////
bool wzHumanPlayer::MakeAMove( wzMove& move )
{
    bool Ret = false;

    // ignore hint request, if any, made before the human 
    // player's turn, which makes the response more intuitive.
    m_pGame->ResetHint();

    while( m_pGame->KeepRunning() )
    {
        // wait for either mouse clicking or typing x, y
        if( !m_pBoard->WaitForInput( move.X, move.Y ) )
        {
            m_pGame->StopGame();
            break;
        }

        // game state may be changed during waiting.
        if( !m_pGame->KeepRunning() )
            break;

        if( m_pGame->NeedHint() )
        {
            GetAHint();
        }
        else
        {
            move.Stone = m_Stone;
            Ret = true;
            break;
        }
    }

    return Ret;
}

//////////////////////////////
void wzHumanPlayer::GetAHint()
{
    wzMove hint;
    hint.Stone = m_Stone;
    m_pAIEngine->BestMove( m_pGame, m_pBoard, hint );
    m_pBoard->SetHint( hint );

    m_pGame->ResetHint();
}

