// wzOptions.cpp - Game options
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzOptions.h"

#include <sstream>
#include <fstream>
using namespace std;


///////////////////////
wzOptions::wzOptions()
{
    SetDefault();
}

//////////////////////////////////////////
void wzOptions::SetDefault()
{
    m_BlackStone = wzCOMPUTER_PLAYER + wzCOMPUTER_LEVEL1;
    m_BlackName  = ComputerPlayerName( wzCOMPUTER_LEVEL1 );

    m_WhiteStone = wzHUMAN_PLAYER;
    m_WhiteName  = wzHUMAN_PLAYER_NAME;
}

//////////////////////////////////////////////////////
wzOptions& wzOptions::operator=( const wzOptions& other )
{
    if( this == &other )
        return( *this );

    m_BlackStone = other.m_BlackStone;
    m_WhiteStone = other.m_WhiteStone;
    m_BlackName  = other.m_BlackName;
    m_WhiteName  = other.m_WhiteName;

    return( *this );
}

//////////////////
// validate 
/////////////////////////////////////////////////////////
bool wzOptions::Valid() const
{
    return( Valid( m_BlackStone, m_BlackName ) && Valid( m_WhiteStone, m_WhiteName ) );
}

/////////////////////////////////////////////////////////
bool wzOptions::Valid( const wzUInt8& code, const string& name )
{
    if( name.empty() ) 
        return false;

    if( (code & wzCOMPUTER_PLAYER) == wzCOMPUTER_PLAYER )
    {
        wzUInt8 level = code ^ wzCOMPUTER_PLAYER;
        if( level >= wzCOMPUTER_LEVEL1 && 
            level <= wzCOMPUTER_MAX_LEVEL )
            return true;
        else
            return false;
    }

    if( code == wzHUMAN_PLAYER ) 
        return true;

    if( code == wzLUA_AI_PLAYER )
        return true;

    return false;
}

///////////////////////////////////////////////
bool wzOptions::HasHumanPlayer() const
{
    return( m_BlackStone == wzHUMAN_PLAYER || m_WhiteStone == wzHUMAN_PLAYER );
}

///////////////////////
// copy options of black stone
//////////////////////////////////////////////////
void wzOptions::CopyBlack( const wzOptions& other )
{
    m_BlackStone = other.m_BlackStone;
    m_BlackName  = other.m_BlackName;
}

///////////////////////
// copy options of white stone
//////////////////////////////////////////////////
void wzOptions::CopyWhite( const wzOptions& other )
{
    m_WhiteStone = other.m_WhiteStone;
    m_WhiteName  = other.m_WhiteName;
}

//////////////////////////////////////////////////
string wzOptions::ComputerPlayerName( const wzUInt8& level )
{
    stringstream ss;
    ss << wzCOMPUTER_PLAYER_NAME << " (L" << static_cast<int>( level ) << ")";

    return ss.str();
}

//////////////////////////////////////////////////
ifstream& operator>>( ifstream& iFile, wzOptions& options )
{
    options.m_BlackStone = iFile.get();
    getline( iFile, options.m_BlackName, '\n' );

    options.m_WhiteStone = iFile.get();
    getline( iFile, options.m_WhiteName, '\n' );

    return iFile;
}

//////////////////////////////////////////////////
ofstream& operator<<( ofstream& oFile, const wzOptions& options )
{
    oFile << options.m_BlackStone;
    oFile << options.m_BlackName << '\n';

    oFile << options.m_WhiteStone;
    oFile << options.m_WhiteName << '\n';

    return oFile;
}
