// wzPattern.cpp - Bitboards, each has a 5-in-a-row
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzPattern.h"
#include "wzPatternIterator.h"


///////////////////////////////
wzPattern::~wzPattern() 
{
    if( m_pIterator != NULL) 
    {
        delete m_pIterator; 
        m_pIterator = NULL; 
    }
}


///////////////////////////////
wzHorizontal5::wzHorizontal5()
  : wzPattern()
{
    // Use only part of the array: (wzDimension-4) * wzDimension
    for( short y = 0; y < wzDimension; y++ )
    {
        for( short x = 0; x < wzDimension-4; x++ )
        {
           for( int i = 0; i < 5; i++ )
               m_BitPatterns[x][y].Set( x + i, y );
        }
    }
}

/////////////////////////////////////////
wzPatternIterator& wzHorizontal5::GetIterator()
{
    if( m_pIterator == NULL) 
    {
        m_pIterator = new wzHorizontal5Iterator( this ); 
    }

    return( *m_pIterator );
}

///////////////////////////////
wzVertical5::wzVertical5()
  : wzPattern()
{
    // Use only part of the array: wzDimension * (wzDimension-4)
    for( short y = 0; y < wzDimension - 4; y++ )
    {
        for( short x = 0; x < wzDimension; x++ )
        {
           for( int i = 0; i < 5; i++ )
               m_BitPatterns[x][y].Set( x, y + i );
        }
    }
}

/////////////////////////////////////////////
wzPatternIterator& wzVertical5::GetIterator()
{
    if( m_pIterator == NULL) 
    {
        m_pIterator = new wzVertical5Iterator( this ); 
    }

    return( *m_pIterator );
}

///////////////////////////////
wzDiagonal5F::wzDiagonal5F()
  : wzPattern()
{ 
    // Use only part of the array: (wzDimension-4) * (wzDimension-4)
    for( short y = 0; y < wzDimension - 4; y++ )
    {
        for( short x = 0; x < wzDimension - 4; x++ )
        {
           for( int i = 0; i < 5; i++ )
               m_BitPatterns[x][y].Set( x + i, y + i );
        }
    }
}

/////////////////////////////////////////////
wzPatternIterator& wzDiagonal5F::GetIterator()
{
    if( m_pIterator == NULL) 
    {
        m_pIterator = new wzDiagonal5FIterator( this ); 
    }

    return( *m_pIterator );
}

///////////////////////////////
wzDiagonal5B::wzDiagonal5B()
  : wzPattern()
{
    // Use only part of the array: (wzDimension-4) * (wzDimension-4)
    for( short y = 0; y < wzDimension - 4; y++ )
    {
        for( short x = 0; x < wzDimension - 4; x++ )
        {
           for( int i = 0; i < 5; i++ )
               m_BitPatterns[x][y].Set( x + 4 - i, y + i);
        }
    }
}

///////////////////////////////////////
wzPatternIterator& wzDiagonal5B::GetIterator()
{
    if( m_pIterator == NULL) 
        m_pIterator = new wzDiagonal5BIterator( this ); 

    return( *m_pIterator );
}

///////////////////////////////
wzStarPattern::wzStarPattern()
{
    for( short y = 0; y < wzDimension; y++ )
    {
        for( short x = 0; x < wzDimension; x++ )
        {
            m_BitPatterns[x][y].Set( x, y );

            short Steps = 2;
            short i, x0, y0;

            // east
            i  = 0; 
            x0 = x - 1;
            while( i < Steps && x0 >= 0 )
            {
                m_BitPatterns[x][y].Set( x0, y );
                i++;
                x0--;
            }

            // west
            i  = 0; 
            x0 = x + 1;
            while( i < Steps && x0 < wzDimension )
            {
                m_BitPatterns[x][y].Set( x0, y );
                i++;
                x0++;
            }

            // north
            i  = 0; 
            y0 = y - 1;
            while( i < Steps && y0 >= 0 )
            {
                m_BitPatterns[x][y].Set( x, y0 );
                i++;
                y0--;
            }

            // south
            i  = 0; 
            y0 = y + 1;
            while( i < Steps && y0 < wzDimension )
            {
                m_BitPatterns[x][y].Set( x, y0 );
                i++;
                y0++;
            }

            // north-west
            i  = 0; 
            x0 = x - 1;
            y0 = y - 1;
            while( i < Steps && x0 >= 0 && y0 >= 0 )
            {
                m_BitPatterns[x][y].Set( x0, y0 );
                i++;
                x0--;
                y0--;
            }

            // south-east
            i  = 0; 
            x0 = x + 1;
            y0 = y + 1;
            while( i < Steps && x0 < wzDimension && y0 < wzDimension )
            {
                m_BitPatterns[x][y].Set( x0, y0 );
                i++;
                x0++;
                y0++;
            }

            // south-west
            i  = 0; 
            x0 = x - 1;
            y0 = y + 1;
            while( i < Steps && x0 >= 0 && y0 < wzDimension )
            {
                m_BitPatterns[x][y].Set( x0, y0 );
                i++;
                x0--;
                y0++;
            }

            // north-east
            i  = 0; 
            x0 = x + 1;
            y0 = y - 1;
            while( i < Steps && x0 < wzDimension && y0 >= 0  )
            {
                m_BitPatterns[x][y].Set( x0, y0 );
                i++;
                x0++;
                y0--;
            }
        }
    }
}
