// wzPattern.h - Bitboards, each has a 5-in-a-row
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_PATTERN_H__
#define __WZ_PATTERN_H__

#include "wzBitBoard.h"
class wzPatternIterator;

#ifndef NULL
#define NULL 0
#endif


///////////////////////////////////////
// Interface represents winning patterns.
//   An array of wzBitBoard, each element has a 5-in-row bits set.
/////////////////////////////////////////////////////////////////////
class wzPattern
{
    friend class wzPatternIterator;

public:
    wzPattern() : m_pIterator( NULL ) {}
    virtual ~wzPattern();

    virtual wzPatternIterator&  GetIterator() = 0;

protected:
    wzBitBoard          m_BitPatterns[wzDimension][wzDimension];
    wzPatternIterator*  m_pIterator;
};

///////////////////////
// horizontal 5-in-a-row
////////////////////////////////////////////////////
class wzHorizontal5 : public wzPattern
{
public:
    wzHorizontal5();
    virtual ~wzHorizontal5() {}

    virtual wzPatternIterator&  GetIterator();
};


///////////////////////
// vertical 5-in-a-row
////////////////////////////////////////////////////
class wzVertical5 : public wzPattern
{
public:
    wzVertical5();
    virtual ~wzVertical5() {}

    virtual wzPatternIterator& GetIterator();
};


///////////////////////
// siagonal 5-in-a-row, forward
////////////////////////////////////////////////////
class wzDiagonal5F : public wzPattern
{
public:
    wzDiagonal5F();
    virtual ~wzDiagonal5F() {}

    virtual wzPatternIterator& GetIterator();
};


///////////////////////
// siagonal 5-in-a-row, backward
////////////////////////////////////////////////////
class wzDiagonal5B : public wzPattern
{
public:
    wzDiagonal5B();
    virtual ~wzDiagonal5B() {}

    virtual wzPatternIterator& GetIterator();
};

///////////////////////
// Pattern of four lines of 5-in-a-row
//   O+O+O
//   +OOO+
//   OOOOO
//   +OOO+
//   O+O+O
////////////////////////////////////////////////////
class wzStarPattern
{
public:
    wzStarPattern();
    virtual ~wzStarPattern() {}

    const wzBitBoard& GetPattern( short x, short y ) { return m_BitPatterns[x][y]; }

private:
    wzBitBoard  m_BitPatterns[wzDimension][wzDimension];
};

#endif
