// wzPatternIterator.h - Iterator for wzPattern
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef __WZ_PATTERN_ITERATOR_H__
#define __WZ_PATTERN_ITERATOR_H__

#include "wzPattern.h"
#include "wzBound.h"


////////////////////////////
class wzPatternIterator
{
public:
    wzPatternIterator( wzPattern* pBitPattern );
    virtual ~wzPatternIterator() {}

    virtual wzBound  GetRow() const = 0;
    virtual bool     SetBound( wzBound& bound );
    virtual bool     SetCenter( short x, short y ) = 0;

    void  Reset();
    bool  Next();
    const wzBitBoard&  GetPattern() { return m_pBitPattern->m_BitPatterns[m_x][m_y]; }

protected:
    short  m_xStart, m_xEnd, m_yStart, m_yEnd;   // bounding box
    short  m_Width, m_Height;                // dimension of bounding box
    short  m_x, m_y;                         // indecies to traverse all patterns

    wzPattern*  m_pBitPattern;

    // function pointer to folowing steppers
    typedef bool (wzPatternIterator::*NextFunc)();
    bool    NextDiagonalF();
    bool    NextDiagonalB();
    bool    NextRectangular();

    NextFunc  m_pNextFunc;

private:
    // forbid: copy and assign
    wzPatternIterator( const wzPatternIterator& other );
    wzPatternIterator& operator=( const wzPatternIterator& other );
};

////////////////////////////
class wzHorizontal5Iterator : public wzPatternIterator
{
public:
    wzHorizontal5Iterator( wzPattern* pBitPattern );
    virtual ~wzHorizontal5Iterator() {}

    virtual wzBound  GetRow() const;
    virtual bool     SetBound( wzBound& bound );
    virtual bool     SetCenter( short x, short y );

private:
    // forbid: copy and assign
    wzHorizontal5Iterator( const wzHorizontal5Iterator& other );
    wzHorizontal5Iterator& operator=( const wzHorizontal5Iterator& other );
};

////////////////////////////
class wzVertical5Iterator : public wzPatternIterator
{
public:
    wzVertical5Iterator( wzPattern* pBitPattern );
    virtual ~wzVertical5Iterator() {}

    virtual wzBound  GetRow() const;
    virtual bool     SetBound( wzBound& bound );
    virtual bool     SetCenter( short x, short y );

private:
    // forbid: copy and assign
    wzVertical5Iterator( const wzVertical5Iterator& other );
    wzVertical5Iterator& operator=( const wzVertical5Iterator& other );
};

////////////////////////////
class wzDiagonal5FIterator : public wzPatternIterator
{
public:
    wzDiagonal5FIterator( wzPattern* pBitPattern );
    virtual ~wzDiagonal5FIterator() {}

    virtual wzBound  GetRow() const;
    virtual bool     SetBound( wzBound& bound );
    virtual bool     SetCenter( short x, short y );

private:
    // forbid: copy and assign
    wzDiagonal5FIterator( const wzDiagonal5FIterator& other );
    wzDiagonal5FIterator& operator=( const wzDiagonal5FIterator& other );
};

////////////////////////////
class wzDiagonal5BIterator : public wzPatternIterator
{
public:
    wzDiagonal5BIterator( wzPattern* pBitPattern );
    virtual ~wzDiagonal5BIterator() {}

    virtual wzBound  GetRow() const;
    virtual bool     SetBound( wzBound& bound );
    virtual bool     SetCenter( short x, short y );

private:
    // forbid: copy and assign
    wzDiagonal5BIterator( const wzDiagonal5BIterator& other );
    wzDiagonal5BIterator& operator=( const wzDiagonal5BIterator& other );
};


#endif
