// wzPlayerFactory.cpp - Player factory base
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzPlayerFactory.h"
#include "wzOptions.h"
#include "wzAIPlayer.h"
#include "wzHumanPlayer.h"


///////////////////////////////
wzPlayer* wzPlayerFactory::Create( wzSTONE stone, const wzOptions& options )
{
    wzUInt8      PlayerCode;
    std::string  PlayerName;

    if( stone == wzBLACK )
    {
        PlayerCode = options.m_BlackStone;
        PlayerName = options.m_BlackName;
    }
    else
    {
        PlayerCode = options.m_WhiteStone;
        PlayerName = options.m_WhiteName;
    }

    if( (PlayerCode & wzCOMPUTER_PLAYER) == wzCOMPUTER_PLAYER )
    {
        return( new wzAIPlayer( stone, PlayerCode ^ wzCOMPUTER_PLAYER ) );
    }
    else if( PlayerCode == wzHUMAN_PLAYER )
    {
        return( new wzHumanPlayer( stone ) );
    }
    else if( PlayerCode == wzLUA_AI_PLAYER )
    {
        return CreateLuaPlayer( stone, PlayerName );
    }

    return( new wzPlayer( stone ) );
}

////////////////////////
// create base player
///////////////////////////////////////////////////////////////////
wzPlayer* wzPlayerFactory::CreateLuaPlayer( wzSTONE stone, const std::string& script )
{
    return( new wzPlayer( stone ) );
}
