// wzReferee.cpp - Referee identifies winner of a board position
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzReferee.h"
#include "wzBoard.h"
#include "wzPatternIterator.h"

///////////////
// identify winner
//////////////////////////////////////////////////////////
wzSTONE wzReferee::Winner( wzBoard& b )
{
    wzBound  WinningRow;
    wzMove   LastMove = b.GetLastMove();
    wzSTONE  winner = Winner( b, LastMove, WinningRow );
    if( winner != wzNONE )
        b.SetWinningRow( WinningRow );

    return winner;
}

///////////////
// identify winner and winning row
//     Seach the area affected by last move only
///////////////////////////////////////////////////////////////////////////////////
wzSTONE wzReferee::Winner( const wzBoardBase& b, const wzMove& LastMove )
{
    wzBound  WinningRow;

    return Winner( b, LastMove, WinningRow );
}

///////////////
// identify winner and winning row
//     Seach the area affected by last move only
//////////////////////////////////////////////////////////////////////////
wzSTONE wzReferee::Winner( const wzBoardBase& b, const wzMove& LastMove,
                           wzBound& WinningRow )
{
    if( LastMove.Stone == wzBLACK )
    {
        if( Winning( b.m_BlackStones, m_Horizontal5, LastMove, WinningRow ) )
            return wzBLACK;

        if( Winning( b.m_BlackStones, m_Vertical5, LastMove, WinningRow ) )
            return wzBLACK;

        if( Winning( b.m_BlackStones, m_Diagonal5F, LastMove, WinningRow ) )
            return wzBLACK;

        if( Winning( b.m_BlackStones, m_Diagonal5B, LastMove, WinningRow ) )
            return wzBLACK;
    }
    else if( LastMove.Stone == wzWHITE )
    {
        if( Winning( b.m_WhiteStones, m_Horizontal5, LastMove, WinningRow ) )
            return wzWHITE;

        if( Winning( b.m_WhiteStones, m_Vertical5, LastMove, WinningRow ) )
            return wzWHITE;

        if( Winning( b.m_WhiteStones, m_Diagonal5F, LastMove, WinningRow ) )
            return wzWHITE;

        if( Winning( b.m_WhiteStones, m_Diagonal5B, LastMove, WinningRow ) )
            return wzWHITE;
    }

    return wzNONE;
}

///////////////
// identify winner and winning row
//     Seach the area affected by last move only
////////////////////////////////////////////////////////////////////////////
bool wzReferee::Winning( const wzBitBoard& bb, wzPattern& pattern, 
                         const wzMove& LastMove, wzBound& WinningRow )
{
    wzPatternIterator& it = pattern.GetIterator();

    // nothing to search
    if( !it.SetCenter( LastMove.X, LastMove.Y ) )
        return false;

    do
    {
        if( (bb & it.GetPattern()) == 5 )
        {
            WinningRow = it.GetRow();
            return true;
        }
    } while( it.Next() );

    return false;
}

