// wzLuaBoard.h - Board for Lua player
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_LUA_BOARD_H__
#define __WZ_LUA_BOARD_H__

extern "C" {
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
}

#include "wzBoardBase.h"

class wzMoveObserver;
class wzBoard;


////////////////////
// Board provides basic facility for Lua AI player. 
//////////////////////////////////////////////////////////
class wzLuaBoard : public wzBoardBase
{
public:
    wzLuaBoard( const wzBoard& board );
    virtual ~wzLuaBoard();

public:
    // register this object to Lua
    static void  Register( lua_State* pLS );

    // untilites 
    static int  Cast2Lua( lua_State* pLS, wzLuaBoard* pBoard );

private: 
    wzLuaBoard( const wzLuaBoard& other );
    wzLuaBoard* Clone() { return new wzLuaBoard( *this ); }

private: 
    wzMoveObserver*  m_pMoveObserver;

private: 
    // the following methods will be registered to Lua

    // inquiries
    static int  isempty  ( lua_State* pLS );
    static int  isfull   ( lua_State* pLS );
    static int  inboard  ( lua_State* pLS );
    static int  getplies ( lua_State* pLS );
    static int  getstone ( lua_State* pLS );
    static int  dimension( lua_State* pLS );
    static int  lastmove ( lua_State* pLS );

    // operations
    static int  putstone( lua_State* pLS );
    static int  remove  ( lua_State* pLS );
    static int  clean   ( lua_State* pLS );

    // clone this object
    static int  clone( lua_State* pLS );

private:
    // utilites
    static wzLuaBoard*  ValidBoard( lua_State* pLS, const int& iArg );
    static int          ValidXY   ( lua_State* pLS, const int& iArg );
    static wzSTONE      ValidColor( lua_State* pLS, const int& iArg );

    // garbage collector
    static int  Destroy( lua_State* pLS );

     // names of the methods
    static const luaL_reg  m_Methods[];

    // ID of Lua object of this class
    static const char  m_ClassName[];

private:
    wzLuaBoard();
    wzLuaBoard& operator=( const wzLuaBoard& other );
};

#endif
