// wzLuaPlayer.h - Player of AI built using Lua script
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_LUA_PLAYER_H__
#define __WZ_LUA_PLAYER_H__

extern "C" {
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
}

#include <wzPlayer.h>
#include <string>

class wzLuaBoard;
class wzLuaVM;


/////////////////
// Player of AI built using Lua script
//////////////////////////////////////////
class wzLuaPlayer : public wzPlayer
{
public:
    wzLuaPlayer( wzSTONE stone, const std::string& LuaScript, wzLuaVM* pLuaVM );
    virtual ~wzLuaPlayer();

    // overrides
    virtual void  Init();
    virtual bool  MakeAMove( wzMove& move );
    virtual void  Set( wzGame* pGame, wzBoard* pBoard );

    static void  Register( lua_State* pLS );

private:
    // utilites
    bool  LuaAIMakeAMove( wzLuaBoard* pBoard, int& x, int& y );
    bool  Retrieve( int& x, int& y );
    bool  Valid( const int& x, const int& y, std::string& msg );
    void  ReturnTypeError( const int& xType, const int& yType, std::string& msg );
    void  LuaAIError( std::string& msg );
    void  LuaAIError();

    static bool  LoadLuaAIFunc ( lua_State* pLS );
    static bool  CallLuaAIFunc ( lua_State* pLS, wzLuaBoard* pBoard );
    static int   LoadErrHandler( lua_State* pLS );

private:
    wzLuaVM*     m_pLuaVM;
    std::string  m_LuaScript;

private:
    // global Lua functions
    static int  MyColor( lua_State* pLS );
    static int  KeepRunning( lua_State* pLS );

    // names
    static const char  m_ClassName[];      // name of this class
    static const char  m_LuaAIFunction[];  // name of Lua AI function
    static const char  m_GamePointer[];    // name of wzGame pointer
};


#endif
