// wzLuaVM.cpp - C++ wrapper of lua_State
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzLuaVM.h"


////////////////////////////////////////
wzLuaVM::wzLuaVM( short LuaLibs )
  : m_pLS( lua_open() )
{
    // load standard libs
    if( LuaLibs & BASE_LIB )    luaopen_base  ( m_pLS );
    if( LuaLibs & STRING_LIB )  luaopen_string( m_pLS );
    if( LuaLibs & TABLE_LIB )   luaopen_table ( m_pLS );
    if( LuaLibs & MATH_LIB )    luaopen_math  ( m_pLS );
    if( LuaLibs & DEBUG_LIB )   luaopen_debug ( m_pLS );
    // for both IO and OS
    if( LuaLibs & IO_LIB || LuaLibs & OS_LIB )      
        luaopen_io( m_pLS ); 
}

///////////////////
// implement Lua's _ALERT function
////////////////////////////////////////
void wzLuaVM::CallAlert() 
{
    lua_getglobal( m_pLS, "_ALERT" );
    if( lua_isfunction( m_pLS, -1 ) ) 
    {
        lua_insert( m_pLS, -2 );
        lua_call( m_pLS, 1, 0 );
    }
}

////////////////////////////
// Register the pointer into Lua's registry, 
// so that  static functions can access the object.
//////////////////////////////////////////////////////////
void wzLuaVM::RegPointer( const char* ObjName, void* pObj )
{
    // if already registered
    if( GetPointer( m_pLS, ObjName ) != NULL ) 
        return;

    // register the pointer to Lua registry
    lua_pushstring( m_pLS, ObjName );
    lua_pushlightuserdata( m_pLS, pObj );
    lua_settable( m_pLS, LUA_REGISTRYINDEX );
}

////////////////////////////
// Retrieve the registered pointer
//////////////////////////////////////////////////////////////////
void* wzLuaVM::GetPointer( lua_State* pLS, const char* ObjName )
{
    void* pObj = NULL;

    lua_pushstring( pLS, ObjName );
    lua_gettable  ( pLS, LUA_REGISTRYINDEX ); 
    if( lua_islightuserdata( pLS, -1 ) )
    {
        pObj = lua_touserdata( pLS, -1 );
    }

    lua_pop( pLS, 1 );

    return pObj;
}
