// wzLuaVM.h - Simple wrapper of lua_State
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_LUA_VM_H__
#define __WZ_LUA_VM_H__

extern "C" {
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
}

#include <string>

class wzLuaBoard;

typedef void (*RegFunc)( lua_State* );


//////////////////////
class wzLuaVM
{
public:
    enum LUALIB { BASE_LIB   = 0x01,    // basic library  
                  STRING_LIB = 0x02,    // string manipulation
                  TABLE_LIB  = 0x04,    // table manipulation
                  MATH_LIB   = 0x08,    // mathematical functions
                  IO_LIB     = 0x10,    // input and output
                  OS_LIB     = 0x20,    // operating system facilities
                  DEBUG_LIB  = 0x40,    // debug facilities.
                  ALL_LIBS   = 0x7F };
                  
    wzLuaVM( short LuaLibs = ALL_LIBS );
    virtual ~wzLuaVM() { lua_close( m_pLS ); }
    
    void  Register ( RegFunc Reg ); 
    bool  RunScript( const char* pScriptName );

    void  CallAlert();

    // register/retrieve pointers of C++ object
    void          RegPointer( const char* ObjName, void* pObj );
    static void*  GetPointer( lua_State* pLS, const char* ObjName );

    // the Lua State
    lua_State* m_pLS;
};

//////////////////////
// other objects call this function to register themselves in to Lua
////////////////////////////////////////////////
inline void wzLuaVM::Register( RegFunc Reg )
{
    Reg( m_pLS );
}

//////////////////////
// run the script to check syntax 
////////////////////////////////////////////////
inline bool wzLuaVM::RunScript( const char* pScriptName )
{
    return( lua_dofile( m_pLS, pScriptName) == 0 );
}

#endif
