// wzTUIApp.cpp - Textual UI app
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzTUIApp.h"
#include "wzTUIUtils.h"
#include <wzOptions.h>
#include <config.h>

#include <string>
#include <iostream>
using namespace std;


/////////////////////////////////
void wzTUIApp::Run()
{
    do
    {
        short game = SelectGame();
        switch( game )
        {
        case NEW_GAME :
            NewGame();
            break;

        case OPEN_GAME :
            OpenGame();
            break;

        default:
            return;
        }

        SaveGame();

    } while( Continue() );
}

/////////////////////////////////////////
bool wzTUIApp::Continue()
{
    char YN = 'y';

    cout << endl;
    cout << "Do you want to play again?(y/n) ";
    cin  >> YN;
    cout << endl;

    return( YN == 'y' || YN == 'Y' );
}

//////////////////////////////////////////
short wzTUIApp::SelectGame()
{
    cout << endl;
    cout << "Please select:" << endl;
    cout << "    1. Start a new" << endl;
    cout << "    2. Open a saved game" << endl;

    return wzTUIUtils::Select1Digit( 1, 2 );
}

/////////////////////////////////////////
void wzTUIApp::NewGame()
{
    wzOptions options;
    if( !SelectOptions( options, wzBLACK ) )
        return;

    if( !SelectOptions( options, wzWHITE ) )
        return;

    if( !m_Game.SetOptions( options ) )
        return;

    m_Game.NewGame();
}

///////////////////////
void wzTUIApp::OpenGame()
{
    m_Game.OpenGame();
}

////////////////////////////////////////
void wzTUIApp::SaveGame()
{
    if( m_Game.IsSaved() )
        return;

    char YN = 'y';
    cout << "Do you want to save game? (y/n) ";
    cin  >> YN;

    if( YN == 'y' || YN == 'Y' )
    {
        cout << "Saving game ..." << endl;
        m_Game.SaveGame();
    }
}

//////////////////////////////////////////////////////////
bool wzTUIApp::SelectOptions( wzOptions& options, wzSTONE stone )
{
    wzUInt8*  pPlayerCode = &options.m_BlackStone;
    string*   pPlayerName = &options.m_BlackName;
    string    StoneName;

    if( stone == wzBLACK )
    {
        StoneName = "black stone";
    }
    else
    {
        StoneName  = "white stone";
        pPlayerCode = &options.m_WhiteStone;
        pPlayerName = &options.m_WhiteName;
    }

    bool  bRet = true;
    short Player = SelectPlayer( StoneName.c_str() );
    switch( Player )
    {
    case COMPUTER_PLAYER :
        {
            wzUInt8 Level = static_cast<wzUInt8>( SelectLevel() );
            *pPlayerCode = wzCOMPUTER_PLAYER + Level;
            *pPlayerName = wzOptions::ComputerPlayerName( Level );
        }

        break;

    case HUMAN_PLAYER :
        *pPlayerCode = wzHUMAN_PLAYER;
        cout << "    Type in player's name: ";
        cin  >> *pPlayerName;

        break;

    case LUA_AI_PLAYER : 
        *pPlayerCode = wzLUA_AI_PLAYER;
        cout << "    Type in the name of Lua script: ";
        cin  >> *pPlayerName;

        break;

    default:
        bRet = false;
    }

    cout << endl;

    return bRet;
}

///////////////////////////////////////
short wzTUIApp::SelectPlayer( const char* StoneName )
{
    cout << endl;
    cout << "Select a player for " << StoneName << endl;
    cout << "    1. " << wzCOMPUTER_PLAYER_NAME << endl;
    cout << "    2. Human Player" << endl;
    cout << "    3. Lua Script"   << endl;

    return wzTUIUtils::Select1Digit( 1, 3 );
}

///////////////////////////////////////
short wzTUIApp::SelectLevel()
{
    cout << endl;
    cout << "Select a level for " << wzCOMPUTER_PLAYER_NAME << endl;
    return wzTUIUtils::Select1Digit( static_cast<short>( wzCOMPUTER_LEVEL1 ), 
                                     static_cast<short>( wzCOMPUTER_MAX_LEVEL ) );
}

/////////////////////////////////////////////////////////////////////
void Banner()
{
    cout << wzAPP_NAME << "  Version " << PACKAGE_VERSION << endl;
    cout << wzAPP_COPYRIGHT << endl;
    cout << PACKAGE_BUGREPORT << endl;
    cout << wzAPP_WEBSIT << endl << endl;
    cout << wzAPP_NAME << " comes with ABSOLUTELY NO WARRANTY." << endl;
    cout << "This is free software, and you are welcome to redistribute" << endl;
    cout << "it under GNU General Public License version 2 or any later" << endl;
    cout << "version at your option." << endl;
    cout << endl;
}

////////////////////
// the very main function
///////////////////////////////
int main()
{
    Banner();

    wzTUIApp  App;
    App.Run();

    return 0;
}
