// wzTUIBoardView.cpp - Board visualization for TUI app.
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzTUIBoardView.h"
#include "wzTUIUtils.h"
#include <wzBoard.h>
#include <wzGame.h>

#include <iostream>
using namespace std;


///////////////////////////////////////
void wzTUIBoardView::UpdateView()
{
    DrawBoard();
    DrawClocks();

    if( m_pBoard->HasWinner() )
    {
        HilightWinningRow();
    }
    else 
    {
        if( !m_pBoard->IsEmpty() )
            HilightLastMove();

        if( m_pBoard->HasHint() )
            HilightHint();
    }
}

//////////////////////////////////////////////////
void wzTUIBoardView::DrawBoard()
{
    for( int y = 0; y < wzDimension; ++y )
    {
        // left label
        if( y < 10 ) 
            cout << "  " << y << " ";
        else
            cout << " "  << y << " ";

        // left edge
        wzSTONE stone = m_pBoard->GetStone( 0, y );
        if( stone == wzNONE )
        {
             cout << "+-";
        }
        else
        {
            if( stone == wzBLACK )
                cout << "X-";
            else
                cout << "O-";
        }

        // middle
        for( int x = 1; x < wzDimension-1; ++x )
        {
            stone = m_pBoard->GetStone( x, y );
            if( stone == wzNONE )
            {
                cout << "-+-";
            }
            else
            {
                if( stone == wzBLACK )
                    cout << "-X-";
                else
                    cout << "-O-";
            }
        }

        // right edge
        stone = m_pBoard->GetStone( wzDimension - 1, y );
        if( stone == wzNONE )
        {
            cout << "-+" << endl;
        }
        else
        {
            if( stone == wzBLACK )
                cout << "-X" << endl;
            else
                cout << "-O" << endl;
        }

        // padding vertical bars or draw bottom label
        cout << "   ";
        if( y != wzDimension - 1 ) 
        {
            // vertical bars between two lines
            for( int x = 0; x < wzDimension; ++x )
            {
                cout << " | ";
            }
        }
        else 
        {
            // bottom label
            for( int x = 0; x < wzDimension; ++x )
            {
                if( x < 10 )
                    cout << " " << x << " ";
                else
                    cout << " " << x;
            }
        }

        cout << endl;
    }
}

//////////////////////////////////////////////////
void wzTUIBoardView::DrawClocks()
{
    cout << "    " 
         << "X - " << m_pGame->GetPlayTime( 0 ) 
         << "\t\t" 
         << "O - " << m_pGame->GetPlayTime( 1 ) << endl;
}

//////////////////////////////////////////////////
void wzTUIBoardView::HilightLastMove()
{
    wzMove move = m_pBoard->GetLastMove();
    cout << "    " 
         << "Last move: (" << move.X << "," << move.Y << ")" << endl;
}

//////////////////////////////////////////////////
void wzTUIBoardView::HilightWinningRow()
{
    wzBound WinningRow = m_pBoard->GetWinningRow();
    cout << "    "
         << "Winning row: (" 
         << WinningRow.xMin << "," << WinningRow.yMin << ") ~ (" 
         << WinningRow.xMax << "," << WinningRow.yMax << ")" << endl;
}

//////////////////////////////////////////////////
void wzTUIBoardView::HilightHint()
{
    wzMove move = m_pBoard->GetHint();
    cout << "You may make a move: (" << move.X << "," << move.Y << ")" << endl;
}


//////////////////////////////////////////////////
bool wzTUIBoardView::WaitForInput( short& x, short& y)
{
    while( true )
    {
        cout << "Please type in x- and y-coodinate (separated by space): " << endl;
 
        // users may type in other things than integers   
        if( !(cin >> x >> y) )
        {
            cin.clear();
            return Select();
        }

        if( !wzBoardBase::Valid( x, y ) )
            cout << "Invalid x or y: must be within [0," << wzDimension-1 << "]" << endl;
        else if( !m_pBoard->IsEmpty( x, y ) )
            cout << "Invalid move: (" << x << ", " << y << ") is not empty" << endl;
        else
            break;

        cout << "!!! Please try again !!!" << endl;
    }

    return true;
}

//////////////////////////////////////////////////
bool wzTUIBoardView::Select()
{
    cout << endl;
    cout << "Please select:" << endl;
    cout << "    1. Undo move" << endl;
    cout << "    2. Get a hint" << endl;
    cout << "    3. Stop game" << endl;
    short select = wzTUIUtils::Select1Digit( 1, 3 );

    bool bRet = true;
    switch( select )
    {
    case 1 :
        m_pGame->UndoMove();
        break;

    case 2 :
        m_pGame->HintMove();
        break;

    case 3 :
        m_pGame->StopGame();
        break;

    default :
        bRet = false;
    }

    return bRet;
}
