// wzTUIGame.cpp - Game for textual UI, subclass of wzGame
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzTUIGame.h"
#include "wzBoard.h"
#include "wzTUIBoardView.h"
#include "wzTUIPlayerFactory.h"

#include <signal.h>
#include <iostream>
using namespace std;


///////////////////////////////////////////
wzGame*  wzTUIGame::m_pThisGame = NULL;

///////////////////////////////////////
wzTUIGame::wzTUIGame()
  : wzGame(),
    m_pBoardView( new wzTUIBoardView( this ) )
{
    m_pBoard->SetView( m_pBoardView );
    SetPlayerFactory( new wzTUIPlayerFactory() );

    m_pThisGame = this;
    signal( SIGINT, CtrlCHandler );
}

///////////////////////
wzTUIGame::~wzTUIGame()
{
    delete m_pBoardView; 
}

//////////////////////////
void wzTUIGame::BlackWins()
{
    cout << "    X wins" << endl;
}

/////////////////////////
void wzTUIGame::WhiteWins()
{
     cout << "    O wins" << endl;
}

////////////////////////
void wzTUIGame::GameDraw()
{
    cout << "    Game draw" << endl;
}

////////////////////////////
void wzTUIGame::GameStopped()
{
    cout << "    Game stopped" << endl;
}

///////////////////////////////////////////
void wzTUIGame::StartMove( short iPlayer )
{
    wzGame::StartMove( iPlayer );

    cout << endl;
    if( iPlayer == 0 )
        cout << "X's turn" << endl;
    else
        cout << "O's turn" << endl;
}

//////////////////////////////////////////////////
std::string wzTUIGame::PromptFile( bool bForOpen )
{
    string FileName;
    if( bForOpen )
        cout << "Type in the file name of the saved game:" << endl;
    else
        cout << "Type in a file name to save the game:" << endl;

    cin >> FileName;

    return FileName;
}

///////////////////////////////////////////////
void wzTUIGame::GameFileError( const char* msg )
{
    cout << "--------- !!! ERROR !!! --------" << endl;
    cout << GetFileName() << endl;
    cout << msg << endl;
    cout << "--------------------------------" << endl;
}

///////////////////////////////////////
void wzTUIGame::CtrlCHandler( int )
{
    if( m_pThisGame->IsRunning() )
    {
        m_pThisGame->StopGame();
        signal( SIGINT, CtrlCHandler );
    }
    else
    {
        signal( SIGINT, SIG_DFL );
    }
}
