// wzAboutDlg.cpp - Frame window for wx GUI app
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include "wzAboutDlg.h"
#include "wzGUIStrings.h"
#include "res/wzLogo64.xpm"


// initialize static data member
const wxString wzAboutDlg::m_wzLicense =
    wxString( wxT("This program is free software; you can redistribute it and/or modify\n") ) +
    wxString( wxT("it under the terms of the GNU General Public License as published by\n") ) +
    wxString( wxT("the Free Software Foundation; either version 2 of the License, or\n") ) +
    wxString( wxT("(at your option) any later version.\n\n") ) +
    wxString( wxT("This program is distributed in the hope that it will be useful,\n") ) +
    wxString( wxT("but WITHOUT ANY WARRANTY; without even the implied warranty of\n") ) +
    wxString( wxT("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n") ) +
    wxString( wxT("GNU General Public License for more details.\n\n") ) +
    wxString( wxT("You should have received a copy of the GNU General Public License\n") ) +
    wxString( wxT("along with this program; if not, write to the Free Software\n") ) +
    wxString( wxT("Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA") );

const wxString wzAboutDlg::m_wzBuildDate  = wxString::Format( _(" (Built on %s)"), __DATE__ );
const wxString wzAboutDlg::m_wzInvitation = _("For source code, manual and tutorial");


/////////////////////////////////////
wzAboutDlg::wzAboutDlg( wxWindow* parent )
  : wxDialog( parent, wxID_ANY, _( "About " )+wzAppName )
{
    InitDlg();
    CenterOnParent();
}

/////////////////////////////////////
void wzAboutDlg::InitDlg()
{
    wxStaticBox*    pOuterBox  = new wxStaticBox  ( this, wxID_ANY, wxT("") );
    wxStaticBitmap* pLogo      = new wxStaticBitmap( this, wxID_ANY, wxBitmap(wzLogo64_xpm) );
    wxStaticText*   pAppName   = new wxStaticText( this, wxID_ANY, wzAppNameLong + m_wzBuildDate );
    wxStaticText*   pCopyright = new wxStaticText( this, wxID_ANY, wzAppCopyright );
    wxStaticText*   pInvite    = new wxStaticText( this, wxID_ANY, m_wzInvitation );
    wxStaticText*   pWebsite   = new wxStaticText( this, wxID_ANY, _("visit ") + wzAppWebsite );
    wxTextCtrl*     pLicense   = new wxTextCtrl  ( this, wxID_ANY, m_wzLicense, 
                                                   wxDefaultPosition, wxDefaultSize, 
                                                   wxTE_READONLY|wxTE_MULTILINE|wxHSCROLL );
    pLicense->SetEditable( false );
    pLicense->SetMinSize( wxSize(200, 100) );

    // OK button
    wxButton* pBtnOK = new wxButton( this, wxID_OK, wxT("OK") );

    // lay out ctrls with sizers
    wxBoxSizer* pRootSizer    = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer* pOuterSizer   = new wxStaticBoxSizer( pOuterBox, wxVERTICAL );
    wxBoxSizer* pAppSizer     = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* pAppInfoSizer = new wxBoxSizer( wxVERTICAL );

    pRootSizer->Add( pOuterSizer, 0, wxALL|wxALIGN_CENTER, 10 );
    pRootSizer->Add( pBtnOK,      0, wxBOTTOM|wxLEFT|wxRIGHT|wxALIGN_RIGHT, 10 );

    pOuterSizer->Add( pAppSizer, 0, wxALL|wxALIGN_CENTER, 5 );
    pOuterSizer->Add( pLicense,  0, wxEXPAND|wxALL|wxALIGN_CENTER_VERTICAL, 5 );

    pAppSizer->Add( pLogo,         0, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 5 ); 
    pAppSizer->Add( pAppInfoSizer, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL|wxALL, 5 );

    pAppInfoSizer->Add( pAppName,   0, wxALIGN_LEFT, 0 );
    pAppInfoSizer->Add( pCopyright, 0, wxEXPAND|wxALIGN_LEFT, 0 );
    pAppInfoSizer->Add( pInvite,    0, wxEXPAND|wxALIGN_LEFT|wxTOP, 5 );
    pAppInfoSizer->Add( pWebsite,   0, wxALIGN_LEFT, 0 );

    SetSizerAndFit( pRootSizer );
}
