 // wzAppConfig.cpp - App configuration using wxFileConfig
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wx/fileconf.h>

#include "wzAppConfig.h"
#include "wzGUIStrings.h"
#include <wzOptions.h>


//////////////////////////////
const wxString wzAppConfig::m_FrameXPosition  = wxT( "/FRAME/X" );
const wxString wzAppConfig::m_FrameYPosition  = wxT( "/FRAME/Y" );
const wxString wzAppConfig::m_FrameBigView    = wxT( "/FRAME/BIG_VIEW" );
const wxString wzAppConfig::m_OptionBlackCode = wxT( "/OPTIONS/BLACK_CODE" );
const wxString wzAppConfig::m_OptionBlackName = wxT( "/OPTIONS/BLACK_NAME" );
const wxString wzAppConfig::m_OptionWhiteCode = wxT( "/OPTIONS/WHITE_CODE" );
const wxString wzAppConfig::m_OptionWhiteName = wxT( "/OPTIONS/WHITE_NAME" );
const wxString wzAppConfig::m_LuaOutputXPosition = wxT( "/LUA_OUTPUT/X" );;
const wxString wzAppConfig::m_LuaOutputYPosition = wxT( "/LUA_OUTPUT/Y" );;
const wxString wzAppConfig::m_LuaOutputWidth  = wxT( "/LUA_OUTPUT/WIDTH" );;
const wxString wzAppConfig::m_LuaOutputHeight = wxT( "/LUA_OUTPUT/HEIGHT" );;
const wxString wzAppConfig::m_TickSoundOn     = wxT( "/TICK_SOUND/ON" );;


/////////////////////////
wzAppConfig::wzAppConfig()
  : m_pConfigFile( new wxFileConfig( wzAppName ) )
{
}

/////////////////////////
wzAppConfig::~wzAppConfig()
{
    delete m_pConfigFile;
}

////////////////////////////////////////////////////////////
void wzAppConfig::ReadFrameConfig( wxPoint& position, bool& bBigview )
{
    long  xPos, yPos;
    if( m_pConfigFile->Read( m_FrameXPosition, &xPos ) && 
        m_pConfigFile->Read( m_FrameYPosition, &yPos ) )
    {
        position.x = static_cast<int>( xPos );
        position.y = static_cast<int>( yPos );
    }
    else
    {
        position = wxDefaultPosition;
    }

    m_pConfigFile->Read( m_FrameBigView, &bBigview, true );
}

/////////////////////////////////////////////////////////////////////////
void wzAppConfig::WriteFrameConfig( const wxPoint& position, const bool& bBigview )
{
    long  pos = static_cast<long>( (position.x > 0)? position.x : 0 );
    m_pConfigFile->Write( m_FrameXPosition,  pos );

    pos = static_cast<long>( (position.y > 0)? position.y : 0 );
    m_pConfigFile->Write( m_FrameYPosition,  pos );

    m_pConfigFile->Write( m_FrameBigView, bBigview );
}

//////////////////////////////////////////////////////
void wzAppConfig::ReadGameOptions( wzOptions& options )
{
    long      PlayerCode;
    wxString  PlayerName; 

    m_pConfigFile->Read( m_OptionBlackCode, &PlayerCode );
    m_pConfigFile->Read( m_OptionBlackName, &PlayerName );
    options.m_BlackStone = static_cast<wzUInt8>( PlayerCode );
    options.m_BlackName  = PlayerName.c_str();

    m_pConfigFile->Read( m_OptionWhiteCode, &PlayerCode );
    m_pConfigFile->Read( m_OptionWhiteName, &PlayerName );
    options.m_WhiteStone = static_cast<wzUInt8>( PlayerCode ); 
    options.m_WhiteName  = PlayerName.c_str();

    if( !options.Valid() )
        options.SetDefault();
}

////////////////////////////////////////////////////////////
void wzAppConfig::WriteGameOptions( const wzOptions& options )
{
    long      PlayerCode = static_cast<long>( options.m_BlackStone );
    wxString  PlayerName = options.m_BlackName.c_str(); 
    m_pConfigFile->Write( m_OptionBlackCode, PlayerCode );
    m_pConfigFile->Write( m_OptionBlackName, PlayerName );

    PlayerCode = static_cast<long>( options.m_WhiteStone );
    PlayerName = options.m_WhiteName.c_str(); 
    m_pConfigFile->Write( m_OptionWhiteCode, PlayerCode );
    m_pConfigFile->Write( m_OptionWhiteName, PlayerName );
}

////////////////////////////////////////////////////////////////////
void wzAppConfig::ReadLuaOutputConfig( wxPoint& position, wxSize& size )
{
    long  xPos, yPos;
    if( m_pConfigFile->Read( m_LuaOutputXPosition, &xPos ) && 
        m_pConfigFile->Read( m_LuaOutputYPosition, &yPos ) )
    {
        position.x = static_cast<int>( xPos );
        position.y = static_cast<int>( yPos );
    }
    else
    {
        position = wxDefaultPosition;
    }

    long  width, height;
    if( m_pConfigFile->Read( m_LuaOutputWidth,  &width ) && 
        m_pConfigFile->Read( m_LuaOutputHeight, &height ) )
    {
        size.Set( static_cast<int>( width ), static_cast<int>( height ) );
    }
    else
    {
        size = wxDefaultSize;
    }
}

/////////////////////////////////////////////////////////////////////////////////
void wzAppConfig::WriteLuaOutputConfig( const wxPoint& position, const wxSize& size )
{
    long  pos = static_cast<long>( (position.x > 0)? position.x : 0 );
    m_pConfigFile->Write( m_LuaOutputXPosition,  pos );

    pos = static_cast<long>( (position.y > 0)? position.y : 0 );
    m_pConfigFile->Write( m_LuaOutputYPosition,  pos );

    m_pConfigFile->Write( m_LuaOutputWidth,  static_cast<long>( size.GetWidth() ) );
    m_pConfigFile->Write( m_LuaOutputHeight, static_cast<long>( size.GetHeight() ) );
}

/////////////////////////////////////////////////////
void wzAppConfig::ReadTickSoundConfig( bool& bSoundOn )
{
    m_pConfigFile->Read( m_TickSoundOn, &bSoundOn, true );
}

////////////////////////////////////////////////////////////
void wzAppConfig::WriteTickSoundConfig( const bool& bSoundOn )
{
    m_pConfigFile->Write( m_TickSoundOn, bSoundOn );
}
