// wzEvent.h - User-defined wxWidgets events
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_EVENT_H__
#define __WZ_EVENT_H__


/////////////////
// Event type
///////////////////////////////////////////////////
BEGIN_DECLARE_EVENT_TYPES()
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_GAME_IDLE,   1 )
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_GAME_RUN,    2 )
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_GAME_END,    3 )
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_GAME_REPLAY, 4 )
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_DISPLAY,     5 )
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_LUA_OUTPUT,  6 )
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_FILE_PROMPT, 7 )
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_FILE_CHANGE, 8 )
  DECLARE_LOCAL_EVENT_TYPE( wzEVT_FILE_ERROR,  9 )
END_DECLARE_EVENT_TYPES()


///////////////////////
// event IDs
////////////////////////////////////////////////
enum wzEventID 
{
    // for wzGameEndEvent
    wzGAME_BLACK_WIN,   // game finished and black wins
    wzGAME_WHITE_WIN,   // game finished and white wins
    wzGAME_DRAW,        // game finished and draw
    wzGAME_STOPPED,     // game stopped and unfinished

    // for wzFilePromptEvent
    wzGAME_FILE_OPEN,
    wzGAME_FILE_SAVE
};


////////////////////
// wzGameRunEvent: game enters idle state
//////////////////////////////////////////////////
class wzGameIdleEvent : public wxEvent
{
public:
    wzGameIdleEvent() : wxEvent( 0, wzEVT_GAME_IDLE )  {}
    wzGameIdleEvent( const wzGameIdleEvent& other ) : wxEvent( other ) {}

    virtual ~wzGameIdleEvent() {}

    virtual wxEvent* Clone() const { return new wzGameIdleEvent( *this ); }

    DECLARE_DYNAMIC_CLASS( wzGameIdleEvent );
};


////////////////////
// wzGameRunEvent: game enters running state
//////////////////////////////////////////////////
class wzGameRunEvent : public wxEvent
{
public:
    wzGameRunEvent() : wxEvent( 0, wzEVT_GAME_RUN )  {}
    wzGameRunEvent( const wzGameRunEvent& other ) : wxEvent( other ) {}

    virtual ~wzGameRunEvent() {}

    virtual wxEvent* Clone() const { return new wzGameRunEvent( *this ); }

    DECLARE_DYNAMIC_CLASS( wzGameRunEvent );
};


////////////////////
// wzGameEndEvent: game ended, can be one of the cases shown in wzEventID
///////////////////////////////////////////////////////////////////////////
class wzGameEndEvent : public wxEvent
{
public:
    wzGameEndEvent( int ID = -1 ) : wxEvent( ID, wzEVT_GAME_END )  {}
    wzGameEndEvent( const wzGameEndEvent& other ) : wxEvent( other ) {}

    virtual ~wzGameEndEvent() {}

    virtual wxEvent* Clone() const { return new wzGameEndEvent( *this ); }

    DECLARE_DYNAMIC_CLASS( wzGameEndEvent );
};


////////////////////
// wzGameReplayEvent: game enters replaying state
//////////////////////////////////////////////////
class wzGameReplayEvent : public wxEvent
{
public:
    wzGameReplayEvent() : wxEvent( 0, wzEVT_GAME_REPLAY )  {}
    wzGameReplayEvent( const wzGameReplayEvent& other ) : wxEvent( other ) {}

    virtual ~wzGameReplayEvent() {}

    virtual wxEvent* Clone() const { return new wzGameReplayEvent( *this ); }

    DECLARE_DYNAMIC_CLASS( wzGameReplayEvent );
};


/////////////////////////
// wzDisplayEvent: board view needs update
////////////////////////////////////////////////////////
class wzDisplayEvent : public wxEvent
{
public:
    wzDisplayEvent() : wxEvent( 0, wzEVT_DISPLAY )  {}
    wzDisplayEvent( const wzDisplayEvent& other ) : wxEvent( other ) {}

    virtual ~wzDisplayEvent() {}

    virtual wxEvent* Clone() const { return new wzDisplayEvent( *this ); }

    DECLARE_DYNAMIC_CLASS( wzDisplayEvent );
};


/////////////////////////
// wzLuaOutputEvent: LuaVM has output 
////////////////////////////////////////////////////////
class wzLuaOutputEvent : public wxEvent
{
public:
    wzLuaOutputEvent() : wxEvent( 0, wzEVT_LUA_OUTPUT )  {}
    wzLuaOutputEvent( const wxString& msg ) : wxEvent( 0, wzEVT_LUA_OUTPUT ), m_Msg( msg ) {}
    wzLuaOutputEvent( const wzLuaOutputEvent& other ) : wxEvent( other ), m_Msg( other.m_Msg ) {}

    virtual ~wzLuaOutputEvent() {}

    virtual wxEvent* Clone() const { return new wzLuaOutputEvent( *this ); }

    void      SetMessage( wxString& msg )  { m_Msg = msg; }
    wxString  GetMessage() const           { return m_Msg; }

private:
    wxString m_Msg;

    DECLARE_DYNAMIC_CLASS( wzLuaOutputEvent );
};


/////////////////////////
// wzFilePromptEvent: prompt for game file name
////////////////////////////////////////////////////////
class wzFilePromptEvent : public wxEvent
{
public:
    wzFilePromptEvent( int ID = -1 ) : wxEvent( ID, wzEVT_FILE_PROMPT )  {}
    wzFilePromptEvent( const wzFilePromptEvent& other ) : wxEvent( other ) {}

    virtual ~wzFilePromptEvent() {}

    virtual wxEvent* Clone() const  { return new wzFilePromptEvent( *this ); }

    DECLARE_DYNAMIC_CLASS( wzFilePromptEvent );
};


/////////////////////////
// wzFileChangeEvent: game file change, can be one the cases defined in wzEventID
//////////////////////////////////////////////////////////////////////////////////
class wzFileChangeEvent : public wxEvent
{
public:
    wzFileChangeEvent() : wxEvent( 0, wzEVT_FILE_CHANGE )  {}
    wzFileChangeEvent( const wzFileChangeEvent& other ) : wxEvent( other ) {}

    virtual ~wzFileChangeEvent() {}

    virtual wxEvent* Clone() const  { return new wzFileChangeEvent( *this ); }

    DECLARE_DYNAMIC_CLASS( wzFileChangeEvent );
};


/////////////////////////
// wzFileErrorEvent: game file I/O error message
////////////////////////////////////////////////////////////////
class wzFileErrorEvent : public wxEvent
{
public:
    wzFileErrorEvent() : wxEvent( 0, wzEVT_FILE_ERROR )  {}
    wzFileErrorEvent( const wxString& msg ) : wxEvent( 0, wzEVT_FILE_ERROR ), m_Msg( msg ) {}
    wzFileErrorEvent( const wzFileErrorEvent& other ) : wxEvent( other ), m_Msg( other.m_Msg ) {}

    virtual ~wzFileErrorEvent() {}

    virtual wxEvent* Clone() const { return new wzFileErrorEvent( *this ); }

    void      SetMessage( wxString& msg )  { m_Msg = msg; }
    wxString  GetMessage() const           { return m_Msg; }

private:
    wxString m_Msg;

    DECLARE_DYNAMIC_CLASS( wzFileErrorEvent );
};


////////////////////////
// handler protptype
//////////////////////////////////////////////
typedef void ( wxEvtHandler::*wzGameIdleHandler )   ( wzGameIdleEvent&   );
typedef void ( wxEvtHandler::*wzGameRunHandler )    ( wzGameRunEvent&    );
typedef void ( wxEvtHandler::*wzGameEndHandler )    ( wzGameEndEvent&    );
typedef void ( wxEvtHandler::*wzGameReplayHandler ) ( wzGameReplayEvent& );
typedef void ( wxEvtHandler::*wzDisplayHandler )    ( wzDisplayEvent&    );
typedef void ( wxEvtHandler::*wzLuaOutputHandler )  ( wzLuaOutputEvent&  );
typedef void ( wxEvtHandler::*wzFilePromptHandler ) ( wzFilePromptEvent& );
typedef void ( wxEvtHandler::*wzFileChangeHandler ) ( wzFileChangeEvent& );
typedef void ( wxEvtHandler::*wzFileErrorHandler )  ( wzFileErrorEvent& );


/////////////////////////
// macro to handling event
/////////////////////////////////////////////////////
#define EVT_WZ_GAME_IDLE( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_GAME_IDLE, 0, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzGameIdleHandler)& fn, \
        (wxObject *)NULL ),

#define EVT_WZ_GAME_RUN( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_GAME_RUN, 0, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzGameRunHandler)& fn, \
        (wxObject *)NULL ),

#define EVT_WZ_GAME_END( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_GAME_END, wxID_ANY, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzGameEndHandler)& fn, \
        (wxObject *)NULL ),

#define EVT_WZ_GAME_REPLAY( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_GAME_REPLAY, 0, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzGameReplayHandler)& fn, \
        (wxObject *)NULL ),

#define EVT_WZ_DISPLAY( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_DISPLAY, 0, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzDisplayHandler)& fn, \
        (wxObject *)NULL ),

#define EVT_WZ_LUA_OUTPUT( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_LUA_OUTPUT, 0, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzLuaOutputHandler)& fn, \
        (wxObject *)NULL ),

#define EVT_WZ_FILE_PROMPT( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_FILE_PROMPT, wxID_ANY, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzFilePromptHandler)& fn, \
        (wxObject *)NULL ),

#define EVT_WZ_FILE_CHANGE( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_FILE_CHANGE, 0, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzFileChangeHandler)& fn, \
        (wxObject *)NULL ),

#define EVT_WZ_FILE_ERROR( fn ) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wzEVT_FILE_ERROR, 0, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)(wzFileErrorHandler)& fn, \
        (wxObject *)NULL ),


#endif  //__WZ_EVENT_H__
