// wzFilePrompter.cpp - Prompt game file
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include "wzFilePrompter.h"
#include "wzEvent.h"
#include "wzGUIStrings.h"


BEGIN_EVENT_TABLE( wzFilePrompter, wxEvtHandler )
    EVT_WZ_FILE_PROMPT( wzFilePrompter::OnFilePrompt )
END_EVENT_TABLE()


//////////////////////////////////////////////
wzFilePrompter::wzFilePrompter()
 :  m_pMutex( new wxMutex() ),
    m_pCondition( new wxCondition( *m_pMutex ) )
{
}

//////////////////////////////////////////////
wzFilePrompter::~wzFilePrompter()
{
    delete m_pCondition;
    delete m_pMutex;
}

/////////////////////////////////////////////////////
std::string wzFilePrompter::PromptFile( bool ForOpen )
{
    // called from main thread
    if( wxThread::IsMain() )
    {
        PopFileDialog( ForOpen );
    }
    else // called from worker thread
    {
        TriggerFileDialog( ForOpen );
        WaitFilePrompt();
    }

    return m_FileName;
}

//////////////////////////////
// send an event to trigger wxFileDialog
/////////////////////////////////////////////////////////
void wzFilePrompter::TriggerFileDialog( bool ForOpen )
{
    wzFilePromptEvent event( wzGAME_FILE_SAVE );
    if( ForOpen )  event.SetId( wzGAME_FILE_OPEN );

    wxPostEvent( this, event );
}

//////////////////////////////////////////////
void wzFilePrompter::OnFilePrompt( wzFilePromptEvent& event )
{
    PopFileDialog( event.GetId() == wzGAME_FILE_OPEN );
    FinishFilePrompt();
}

///////////////////////////////////
// wait for FileDialog done
///////////////////////////////////
void wzFilePrompter::WaitFilePrompt()
{
    wxMutexLocker  lock( *m_pMutex );
    m_pCondition->Wait();
}

///////////////////////////////////
// notify FileDialog done
///////////////////////////////////////////////
void wzFilePrompter::FinishFilePrompt()
{
    wxMutexLocker  lock( *m_pMutex );
    m_pCondition->Signal();
}

/////////////////////////////////////////////////////////////////////
void wzFilePrompter::PopFileDialog( bool bForOpen )
{
    static const wxString Wildcard = wxString( 
           _("Tick5 game files (*.tick5)|*.tick5|All files|*") );

    wxString  Title = wzAppName;
    long      Style;
    if( bForOpen )
    {
        Title += _(" - Select a game file");
        Style = wxOPEN|wxFILE_MUST_EXIST|wxCHANGE_DIR;
    }
    else
    {
        Title += _(" - Save game file");
        Style = wxSAVE|wxOVERWRITE_PROMPT|wxCHANGE_DIR;
    }

    wxFileDialog FileDlg( NULL, Title, wxEmptyString, wxEmptyString, Wildcard, Style );

    if( FileDlg.ShowModal() == wxID_OK )
        m_FileName = FileDlg.GetPath().c_str();
    else
        m_FileName.clear();
}
