// wzGUIBoardView.h - Board view for GUI app.
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_GUI_BOARD_VIEW_H__
#define __WZ_GUI_BOARD_VIEW_H__

#include <wzBoardView.h>

class wzDisplayEvent;


///////////////////
// Board view for GUI app
//////////////////////////////////////////////////////////////
class wzGUIBoardView : public wzBoardView, public wxWindow
{
public:
    wzGUIBoardView( wxWindow* parent );
    virtual ~wzGUIBoardView();

    // view size
    bool   IsBigView() const;
    short  ToggleView();
    short  Width() const;

    // overrides
    virtual void  UpdateView();
    virtual bool  WaitForInput( short& x, short& y);
    virtual void  NotifyInput();

    // event handlers
    void  OnPaint   ( wxPaintEvent& );
    void  OnClick   ( wxMouseEvent& );
    void  OnDisplay ( wzDisplayEvent& );
    void  OnHilight ( wxTimerEvent& );
    void  OnStopHint( wxTimerEvent& );

private:
    bool  WithinView( short x, short y ) const;

    void  BigView();
    void  SmallView();
    void  LoadBigPads();
    void  LoadSmallPads();

    void  PaintBoard( wxDC& dc );
    void  PaintPad( wxDC& dc, wxMemoryDC& mdc, 
                    const short& x, const short& y, wxBitmap pads[3] );
    void  HiglightWinningRow( wxDC& dc );
    void  HiglightLastMove( wxDC& dc );
    void  HiglightHint( wxDC& dc );

    void  BlinkWinningRow( wxDC& dc );
    void  BlinkLastMove( wxDC& dc );
    void  BlinkHint( wxDC& dc );

    void  RefreshWinningRow();
    void  RefreshLastMove();
    void  RefreshHint();

    bool  TimeToPaint();
    void  KeepBlinking( const int& millisecs );

private:
    // mouse clicking
    short         m_xClick;
    short         m_yClick;
    wxCondition*  m_pCondition;    // signal for wzHumanPlayer
    wxMutex*      m_pMutex;

    // dimensions
    short  m_PadWidth;    // width of graphic pads
    short  m_ViewWidth;
    short  m_Half; 
    short  m_One3rd; 
    short  m_WidePen;
    short  m_NarrowPen;

    // blink higlights
    enum { HALF_SECOND = 500, ONE_SECOND = 1000, FIVE_SECONDS = 5000 };
    wxTimer  m_HilightTimer;
    wxTimer  m_HintStopper;
    bool     m_PaintHilight;

    // board pads
    enum { EMPTY = 0, BALCK, WHITE };

    wxBitmap  m_PadsTopLeft[3];
    wxBitmap  m_PadsTopMiddle[3];
    wxBitmap  m_PadsTopRight[3];

    wxBitmap  m_PadsCentralLeft[3];
    wxBitmap  m_PadsCentralMiddle[3];
    wxBitmap  m_PadsCentralRight[3];

    wxBitmap  m_PadsBottomLeft[3];
    wxBitmap  m_PadsBottomMiddle[3];
    wxBitmap  m_PadsBottomRight[3];

private:
    wzGUIBoardView( const wzGUIBoardView& );
    wzGUIBoardView& operator=( const wzGUIBoardView& );

    DECLARE_EVENT_TABLE()
};


/////////////////////
// Display event handler, called by main thread.
///////////////////////////////////////////////
inline void wzGUIBoardView::OnDisplay( wzDisplayEvent& event )
{
    Refresh();
}

///////////////////////////////////
inline short wzGUIBoardView::Width() const
{ 
    return m_ViewWidth;
}

///////////////////////////////////////
inline bool wzGUIBoardView::IsBigView() const 
{ 
    return( m_PadWidth == 40 ); 
}

///////////////////////
// Need this on wxGTK. 
// On wxGTK, the view still gets wxMouseEvent, even though
// mouse is clicked on the view and released outside of the 
// frame window.
////////////////////////////////////////////////////////////////
inline bool wzGUIBoardView::WithinView( short x, short y ) const
{
    return( x >= 0 && x < m_ViewWidth && y >= 0 && y < m_ViewWidth ); 
}

#endif
