// wzGUIFrame.h - Frame window for wx GUI app
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_GUI_FRAME_H__
#define __WZ_GUI_FRAME_H__

class wzGUIGame;
class wzGUIBoardView;
class wzGameIdleEvent;
class wzGameRunEvent;
class wzGameEndEvent;
class wzGameReplayEvent;
class wzLuaOutputEvent;
class wzFileChangeEvent;
class wzFileErrorEvent;
class wzLuaOutputDlg;
class wzOptionsDlg;


////////////////////
// Frame 
/////////////////////////////////////
class wzGUIFrame: public wxFrame
{
public:
    wzGUIFrame();
    virtual ~wzGUIFrame();

    void  OnNewGame   ( wxCommandEvent& event );
    void  OnOpenGame  ( wxCommandEvent& event );
    void  OnSaveGame  ( wxCommandEvent& event );
    void  OnSaveGameAs( wxCommandEvent& event );
    void  OnStopGame  ( wxCommandEvent& event );
    void  OnUndoMove  ( wxCommandEvent& event );
    void  OnHintMove  ( wxCommandEvent& event );
    void  OnFirstPly  ( wxCommandEvent& event );
    void  OnPrevPly   ( wxCommandEvent& event );
    void  OnNextPly   ( wxCommandEvent& event );
    void  OnLastPly   ( wxCommandEvent& event );
    void  OnOptions   ( wxCommandEvent& event );
    void  OnToggleView( wxCommandEvent& event );
    void  OnToggleSound( wxCommandEvent& event );
    void  OnAbout     ( wxCommandEvent& event );
    void  OnClose     ( wxCloseEvent&   event );
    void  OnGameRun   ( wzGameRunEvent& event );
    void  OnGameEnd   ( wzGameEndEvent& event );
    void  OnGameReplay( wzGameReplayEvent& event );
    void  OnGameIdle  ( wzGameIdleEvent&   event );
    void  OnLuaOutput ( wzLuaOutputEvent&  event );
    void  OnFileChange( wzFileChangeEvent& event );
    void  OnFileError ( wzFileErrorEvent&  event );
    void  OnTimer     ( wxTimerEvent&      event );

private:
    // toolbar and status bar
    void  CreateStatusbar();
    void  CreateToolbar();

    void  IdleStatusbar();
    void  IdleToolbar();

    void  RunningStatusbar();
    void  RunningToolbar();

    void  EndedStatusbar( const int ID );
    void  EndedToolbar();

    void  ReplayingStatusbar();
    void  ReplayingToolbar();

    void  FlipStatusMsg();
    void  UpdateClocks();

    // config file
    void LoadConfigFile();
    void SaveConfigFile();

    // utils
    static wxString StripPath( wxString& filename );

private:
    // game
    wzGUIBoardView*  m_pBoardView;
    wzGUIGame*       m_pGame;

    // GUI componets
    wxToolBar*       m_pToolBar;
    wzLuaOutputDlg*  m_pLuaOutputDlg;

    // status bar msg
    wxString  m_StatusMsg[2];
    wxTimer   m_StatusTimer;
    wxTimer   m_ClockTicker;

    // exit flag
    bool  m_Exit;

    enum { MSG_FIELD = 1, CLOCK_FIELD0, CLOCK_FIELD1 };

    DECLARE_EVENT_TABLE()
};

#endif
