// wzGUIGame.cpp - Game for wx GUI, subclass of wzGame
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wx/filename.h>

#include "wzGUIGame.h"
#include "wzBoard.h"
#include "wzGUIPlayerFactory.h"
#include "wzEvent.h"
#include "wzGUIStrings.h"
#include "wzFilePrompter.h"
#include "wzTickSound.h"


////////////////////////////////////////////////////////////////
wzGUIGame::wzGUIGame( wxFrame* pFrame, wzBoardView* pBoardView  )
  : wzGame(),
    m_pFilePrompter( new wzFilePrompter() ),
    m_pEvtHandler  ( pFrame ),
    m_pTickSound   ( new wzTickSound() )
{
    m_pBoard->SetView( pBoardView );
    m_pBoard->Register( m_pTickSound );
    SetSyncObject( new wzGUISyncObject() );
    SetPlayerFactory( new wzGUIPlayerFactory( pFrame ) );
}

//////////////////////
wzGUIGame::~wzGUIGame()
{
    delete m_pFilePrompter;
    delete m_pTickSound;
}

////////////////////////////
void wzGUIGame::EnableSound( bool bSound )
{
    m_pTickSound->Enable( bSound );
}

////////////////////////////////
// create a thread to run the game.
///////////////////////////////////////////////////////
void wzGUIGame::Run()
{
    wzGameThread* pGameThread = new wzGameThread( this );
    if( pGameThread->Create() == wxTHREAD_NO_ERROR )
    {
        pGameThread->SetPriority( WXTHREAD_MIN_PRIORITY );
        pGameThread->Run();
    }
}

//////////////////////
// thread function
////////////////////////////////////////////////////
wxThread::ExitCode wzGUIGame::wzGameThread::Entry()
{
    m_pGame->RunGame();

    return 0;
}

//////////////////////////
void wzGUIGame::BeUndoState()
{
    wzGame::BeUndoState();
    PhonyClick();
}

//////////////////////////
void wzGUIGame::BeHintState()
{
    wzGame::BeHintState();
    PhonyClick();
}

//////////////////////////
void wzGUIGame::BeSaveState()
{
    wzGame::BeSaveState();
    PhonyClick();
}

//////////////////////////
void wzGUIGame::BeSaveAsState()
{
    wzGame::BeSaveAsState();
    PhonyClick();
}

//////////////////////////
void wzGUIGame::BeEndedState()
{
    wzGame::BeEndedState();
    PhonyClick();
}

//////////////////////
// Fake a mouse click, in case that a wzHumanPlayer running in 
// the worker thread is still waiting for mouse click, 
// which could block the thread.
// It's harmless, if no one is waiting for the signal.
/////////////////////////////////////////////////////////////
void wzGUIGame::PhonyClick()
{
    m_pBoard->NotifyInput();
}

//////////////////////////
void wzGUIGame::GameIdle()
{
    wzGameIdleEvent event;
    wxPostEvent( m_pEvtHandler, event );
}

///////////////////////////
void wzGUIGame::BlackWins()
{
    wzGameEndEvent event( wzGAME_BLACK_WIN );
    wxPostEvent( m_pEvtHandler, event );
}

////////////////////////////
void wzGUIGame::WhiteWins()
{
    wzGameEndEvent event( wzGAME_WHITE_WIN );
    wxPostEvent( m_pEvtHandler, event );
}

///////////////////////////
void wzGUIGame::GameDraw()
{
    wzGameEndEvent event( wzGAME_DRAW );
    wxPostEvent( m_pEvtHandler, event );
}

///////////////////////////////
void wzGUIGame::GameStopped()
{
    wzGameEndEvent event( wzGAME_STOPPED );
    wxPostEvent( m_pEvtHandler, event );
}

/////////////////////////////
void wzGUIGame::GameRunning()
{
    wzGameRunEvent event;
    wxPostEvent( m_pEvtHandler, event );
}

////////////////////////////////
void wzGUIGame::GameReplaying()
{
    wzGameReplayEvent event;
    wxPostEvent( m_pEvtHandler, event );
}

/////////////////////////
// Instead of popping up wxMessageBox, post an event,
// because this function can be called from worker thread.
///////////////////////////////////////////////////////////
void wzGUIGame::GameFileError( const char* ErrorMsg )
{
    wxString  Msg( ErrorMsg );

    wxString  FileName;
    wxString  Path = GetFileName().c_str();
    if( !Path.IsEmpty() )
    {
        wxFileName FullPath( Path );
        FileName = FullPath.GetFullName();

        Msg = FileName + _( "    \n\n" ) + Msg;
    }

    wzFileErrorEvent event( Msg );
    wxPostEvent( m_pEvtHandler, event );
}

////////////////////////////////////////////////
void wzGUIGame::GameFileChange()
{
    wzFileChangeEvent event;
    wxPostEvent( m_pEvtHandler, event );
}

//////////////////////////////////////////////
std::string wzGUIGame::PromptFile( bool bForOpen )
{
    return( m_pFilePrompter->PromptFile( bForOpen ) );
}
