// wzGUIGame.h - Game for wx GUI, subclass of wzGame
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_GUI_GAME_H__
#define __WZ_GUI_GAME_H__

#include "wzGame.h"

class wzFilePrompter;
class wzBoardView;
class wzTickSound;


/////////////////////
// Graphic UI game
////////////////////////////////////////////////
class wzGUIGame : public wzGame
{
public:
    wzGUIGame( wxFrame* pFrame, wzBoardView* pBoardView );
    virtual ~wzGUIGame();

    void  EnableSound( bool bSound );

protected:
    // overrides
    virtual void  Run();
    virtual void  BeSaveState();
    virtual void  BeSaveAsState();
    virtual void  BeUndoState();
    virtual void  BeHintState();
    virtual void  BeEndedState();

    virtual void  GameIdle();
    virtual void  BlackWins();
    virtual void  WhiteWins();
    virtual void  GameDraw();
    virtual void  GameStopped();
    virtual void  GameRunning();
    virtual void  GameReplaying();
    virtual void  GameFileError( const char* msg );
    virtual void  GameFileChange();

    virtual std::string  PromptFile( bool bOpen );

    class wzGUISyncObject : public wzGame::wzSyncObject
    {
    public:
        wzGUISyncObject()           {}
        virtual ~wzGUISyncObject()  {}

        virtual void  Lock()   { m_CritSect.Enter(); }
        virtual void  Unlock() { m_CritSect.Leave(); }

    private:
        wxCriticalSection  m_CritSect;
    };

private:
    void  RunGame();
    void  PhonyClick();

    class wzGameThread : public wxThread
    {
    public:
        wzGameThread( wzGUIGame* pGame ) : m_pGame( pGame ) {}
        virtual ~wzGameThread() {}
       
        virtual ExitCode  Entry();

    private:
        wzGUIGame*  m_pGame;
    };

private:
    wzFilePrompter*  m_pFilePrompter;
    wxEvtHandler*    m_pEvtHandler;
    wzTickSound*     m_pTickSound;
};


///////////////////////////////
inline void wzGUIGame::RunGame() 
{ 
    wzGame::Run(); 
}

#endif
