// wzGUILuaVM.cpp - C++ wrapper of lua_State for GUI app
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wzDefs.h>
#include "wzGUILuaVM.h"
#include "wzEvent.h"


const char wzGUILuaVM::m_ClassName[] = "WZ_GUI_LUA_VM";

///////////////////
// replace some of the methods in Lua IO lib
////////////////////////////////////////////////////
const luaL_reg wzGUILuaVM::m_LuaIOMethods[] = 
{
    { "input",  LuaIOHandler },
    { "output", LuaIOHandler },
    { "flush",  LuaIOHandler },
    { "read",   LuaIOHandler },
    { "write",  LuaPrint },
    { NULL, NULL}
};

////////////////////////////////////////
wzGUILuaVM::wzGUILuaVM( wxEvtHandler* pEvtHandler, short LuaLibs )
  : wzLuaVM( LuaLibs ),
    m_pEvtHandler( pEvtHandler )
{
    // Register the pointer of this object
    RegPointer( m_ClassName, this );
    RegMethods();
}

////////////////////////////////////////
void wzGUILuaVM::PostEvent( wxEvent& event )
{
    wxPostEvent( m_pEvtHandler, event );
}

////////////////////////////
// Register these with Lua to replace some Lua functions
//////////////////////////////////////////////////////////////////////
void wzGUILuaVM::RegMethods()
{
    // register it global to replace Lua's print
    lua_register( m_pLS, "print", LuaPrint );

    // register Lua's _ALERT, for error messsags.
    // by default, the method is not implemented, but a placehold
    lua_register( m_pLS, "_ALERT", LuaAlert );

    // replace some functions in Lua's IO lib
    // see m_vpLuaIOFuncs for details
    luaL_openlib( m_pLS, LUA_IOLIBNAME, m_LuaIOMethods, 1 );
}

///////////////
// take care Lua's string outputs
////////////////////////////////////////////////
int wzGUILuaVM::LuaAlert( lua_State* pLS )
{
    wxString msg;
    msg += wxString::Format( wxT("%s\n"), lua_tostring( pLS, -1 ) );
    lua_pop( pLS, 1 ); 

    PostMsg( pLS, msg );
    return 0;
}

void wzGUILuaVM::PostMsg( lua_State* pLS, const wxString& msg )
{
    wzLuaOutputEvent event( msg );
    wzGUILuaVM* pLVM = static_cast<wzGUILuaVM*>( GetPointer( pLS, m_ClassName ) );
    pLVM->PostEvent( event );
}

///////////////
// replace Lua's print
// see example lua-5.0/etc/min.c 
///////////////////////////////////////////////////
int wzGUILuaVM::LuaPrint( lua_State* pLS )
{
    wxString msg;
    int n = lua_gettop( pLS );
    for( int i = 1; i <= n; i++ )
    {
        if( lua_isstring( pLS, i ) )  
            msg += wxString::Format( wxT("%s"), lua_tostring( pLS, i) );
        else if( lua_isnil( pLS , i ) )
            msg += wxString::Format( wxT("%s"), wxT("nil") );
        else if( lua_isboolean( pLS, i ) )
            msg += wxString::Format( wxT("%s"), lua_toboolean( pLS, i )?wxT("true"):wxT("false") );
        else
            msg += wxString::Format( wxT("%s:%p"), lua_typename(pLS , lua_type(pLS, i)), lua_topointer( pLS, i ) );
    }

    msg += wxString( wxT("\n") );

    PostMsg( pLS, msg );
    return 0;
}

///////////////
// replace some of Lua's IO methods
////////////////////////////////////////////////
int wzGUILuaVM::LuaIOHandler( lua_State* pLS )
{
    lua_Debug LuaD;
    lua_getstack( pLS, 0,  &LuaD );
    lua_getinfo( pLS, "n", &LuaD );

    if( LuaD.name == NULL )
        LuaD.name = "?";

    return luaL_error( pLS, "`%s' failed (not supported in %s)", LuaD.name, wzAPP_NAME );
}
