// wzLuaOutputDlg.cpp - Frame window for wx GUI app
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include "wzLuaOutputDlg.h"
#include "wzGUIStrings.h"
#include "res/wzLogo16.xpm"
#include "res/wzLogo32.xpm"


BEGIN_EVENT_TABLE( wzLuaOutputDlg, wxDialog )
    EVT_CLOSE (             wzLuaOutputDlg::OnClose )
    EVT_BUTTON( wxID_CLOSE, wzLuaOutputDlg::OnCloseBtn )
    EVT_BUTTON( wxID_CLEAR, wzLuaOutputDlg::OnClearBtn )
END_EVENT_TABLE()

/////////////////////////////////////
// Create an orphan dialog, so it won't stay on top.
// On wxGTK, it still stays on top, wxMINIMIZE_BOX doesn't take effect. 
/////////////////////////////////////////////////////////
wzLuaOutputDlg::wzLuaOutputDlg()
  : wxDialog( NULL, wxID_ANY, wzAppName + _(" - Lua Output"), 
              wxDefaultPosition, wxDefaultSize, 
              wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER|wxDIALOG_NO_PARENT )
{
    wxIconBundle Icons;
    Icons.AddIcon( wxIcon(wzLogo16_xpm) );
    Icons.AddIcon( wxIcon(wzLogo32_xpm) );
    SetIcons( Icons );

    InitDlg();
}

////////////////////////////////////////////////////
void wzLuaOutputDlg::InitDlg()
{
    m_pOutputMsg = new wxTextCtrl( this, wxID_ANY, wxEmptyString, 
                                   wxDefaultPosition,  wxDefaultSize,
                                   wxTE_READONLY|wxTE_MULTILINE|wxHSCROLL );

    wxButton*  pBtnClear = new wxButton( this, wxID_CLEAR, _("Clear") );
    wxButton*  pBtnClose = new wxButton( this, wxID_CLOSE, _("Close") );

    wxBoxSizer* pTopSizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer* pBtnSizer = new wxBoxSizer( wxHORIZONTAL );

    pTopSizer->Add( m_pOutputMsg, 1, wxEXPAND|wxALIGN_CENTER|wxALL, 6 );
    pTopSizer->Add( pBtnSizer,    0, wxALIGN_RIGHT|wxBOTTOM|wxLEFT|wxRIGHT, 6 );
    pBtnSizer->Add( pBtnClear,    0, wxALIGN_RIGHT, 0 );
    pBtnSizer->Add( pBtnClose,    0, wxLEFT|wxALIGN_RIGHT, 6 );
    SetSizerAndFit( pTopSizer );

    SetMinSize( wxSize(400, 200) );
}

///////////////////////////////
void wzLuaOutputDlg::Append( const wxString& msg )
{
    if( !IsShown() )  Show( true );

    m_pOutputMsg->AppendText( msg );
}
