// wzOptionsDlg.cpp - Game options dialog for wx GUI app
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wx/spinctrl.h>

#include <wzOptions.h>
#include "wzIDs.h"
#include "wzGUIStrings.h"
#include "wzOptionsDlg.h"
#include "res/Black.xpm"
#include "res/White.xpm"
#include "res/wzLogo16.xpm"
#include "res/wzLogo32.xpm"


BEGIN_EVENT_TABLE( wzOptionsDlg, wxDialog )
    EVT_RADIOBUTTON( wzID_BLACK_COMPUTER, wzOptionsDlg::OnBlackRadioBtn )
    EVT_RADIOBUTTON( wzID_BLACK_HUMAN,    wzOptionsDlg::OnBlackRadioBtn )
    EVT_RADIOBUTTON( wzID_BLACK_LUA_AI,   wzOptionsDlg::OnBlackRadioBtn )
    EVT_RADIOBUTTON( wzID_WHITE_COMPUTER, wzOptionsDlg::OnWhiteRadioBtn )
    EVT_RADIOBUTTON( wzID_WHITE_HUMAN,    wzOptionsDlg::OnWhiteRadioBtn )
    EVT_RADIOBUTTON( wzID_WHITE_LUA_AI,   wzOptionsDlg::OnWhiteRadioBtn )
    EVT_SPINCTRL( wzID_BLACK_COMP_LEVEL,  wzOptionsDlg::OnBlackSpinCtrl )
    EVT_SPINCTRL( wzID_WHITE_COMP_LEVEL,  wzOptionsDlg::OnWhiteSpinCtrl )
    EVT_BUTTON  ( wzID_BLACK_LUA_SEL,     wzOptionsDlg::OnBlackSeleBtn  )
    EVT_BUTTON  ( wzID_WHITE_LUA_SEL,     wzOptionsDlg::OnWhiteSeleBtn  )
END_EVENT_TABLE()


////////////////////////////////////////////////////////////////
wzOptionsDlg::wzOptionsDlg( wxWindow* parent, wzOptions& options )
  : wxDialog( parent, wxID_ANY, wzAppName + _(" - Game Options") )
{
    wxIconBundle Icons;
    Icons.AddIcon( wxIcon(wzLogo16_xpm) );
    Icons.AddIcon( wxIcon(wzLogo32_xpm) );
    SetIcons( Icons );

    CreateDlg();
    Init( options );
    CenterOnParent();
}

////////////////////////////////////////////////////////////////
void wzOptionsDlg::CreateDlg()
{
    // static boxes
    CreateBlackBox();
    CreateWhiteBox();
 
    // OK, Cancel buttons
    wxButton* pBtnOK     = new wxButton( this, wxID_OK,     _("OK") );
    wxButton* pBtnCancel = new wxButton( this, wxID_CANCEL, _("Cancel") );

    // lay out ctrls with sizers
    wxBoxSizer*  TopSizer   = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer*  ButtnSizer = new wxBoxSizer( wxHORIZONTAL );

    // black and white box
    TopSizer->Add( m_pBlackBox, 0, wxALIGN_CENTER|wxALL,          10 );
    TopSizer->Add( m_pWhiteBox, 0, wxALIGN_CENTER|wxLEFT|wxRIGHT, 10 );

    // OK, Cancel buttons
    TopSizer->Add  ( ButtnSizer, 0, wxALIGN_RIGHT|wxALL, 10 );
    ButtnSizer->Add( pBtnOK,     0, wxRIGHT, 10 );
    ButtnSizer->Add( pBtnCancel );

    SetSizerAndFit( TopSizer );
}

//////////////////////
// static box for black
/////////////////////////////////////////////////////
void wzOptionsDlg::CreateBlackBox()
{
    wzBoxParameters BlackParams;
    BlackParams.IDComputer = wzID_BLACK_COMPUTER;
    BlackParams.IDLevel    = wzID_BLACK_COMP_LEVEL;
    BlackParams.IDHuman    = wzID_BLACK_HUMAN;
    BlackParams.IDLuaAI    = wzID_BLACK_LUA_AI;
    BlackParams.IDLuaBtn   = wzID_BLACK_LUA_SEL;
    BlackParams.xpm        = Black_xpm;

    m_pBlackBox = new wzOptionBox( this, BlackParams );
}

///////////////////////
// static box for white
////////////////////////////////////////////////////
void wzOptionsDlg::CreateWhiteBox()
{
    wzBoxParameters WhiteParams;
    WhiteParams.IDComputer = wzID_WHITE_COMPUTER;
    WhiteParams.IDLevel    = wzID_WHITE_COMP_LEVEL;
    WhiteParams.IDHuman    = wzID_WHITE_HUMAN;
    WhiteParams.IDLuaAI    = wzID_WHITE_LUA_AI;
    WhiteParams.IDLuaBtn   = wzID_WHITE_LUA_SEL;
    WhiteParams.xpm        = White_xpm;

    m_pWhiteBox = new wzOptionBox( this, WhiteParams );
}

////////////////////////////////////////////////////////////////
void wzOptionsDlg::Init( wzOptions& options )
{
    m_pBlackBox->Init( options.m_BlackStone, options.m_BlackName );
    m_pWhiteBox->Init( options.m_WhiteStone, options.m_WhiteName );
}

////////////////////////////////////////////////////////////////
void wzOptionsDlg::GetOptions( wzOptions& options ) const
{
    m_pBlackBox->GetOptions( options.m_BlackStone, options.m_BlackName );
    m_pWhiteBox->GetOptions( options.m_WhiteStone, options.m_WhiteName );
}

////////////////////////////////////////////////////////////////
wzOptionBox::wzOptionBox( wxWindow* parent, wzBoxParameters& parameters )
  : wxStaticBoxSizer( wxHORIZONTAL, parent ),
    m_Parent( parent )
{
    CreateBox( parameters );
}

////////////////////////////////////////////////////////////////
void wzOptionBox::CreateBox( wzBoxParameters& param )
{
    wxBitmap Stone = wxBitmap( param.xpm );
    wxStaticBitmap* pStoneBitmap = new wxStaticBitmap( m_Parent, wxID_ANY, Stone );

    // computer player
    m_pBtnComputer = new wxRadioButton( m_Parent, param.IDComputer, _(wzCOMPUTER_PLAYER_NAME),
                                        wxDefaultPosition, wxDefaultSize, wxRB_GROUP );
    m_pSpinLevel = new wxSpinCtrl( m_Parent, param.IDLevel, wxEmptyString, 
                                   wxDefaultPosition, wxSize(50, 25) );
    m_pSpinLevel->SetRange( 1, static_cast<int>(wzCOMPUTER_MAX_LEVEL) );

    // human player
    m_pBtnHuman = new wxRadioButton( m_Parent, param.IDHuman, _("Human Player") );
    m_pTxtHuman = new wxTextCtrl( m_Parent, wxID_ANY, wxEmptyString, wxDefaultPosition, 
                                  wxSize( 150, 25 ) );

    // Lua script player
    m_pBtnLuaAI = new wxRadioButton( m_Parent, param.IDLuaAI, _("Lua Script") );
    m_pTxtLuaScript = new wxTextCtrl( m_Parent, wxID_ANY, wxEmptyString, wxDefaultPosition, 
                                      wxSize( 150, 25 ) );
    m_pBtnLuaScript = new wxButton( m_Parent, param.IDLuaBtn, wxT("..."), wxDefaultPosition,
                                    wxSize( 25, 25 ) );

    // grid sizer on the right
    wxGridSizer* pGridSizer = new wxFlexGridSizer( 3, 2, 5, 5 );

    // sizer to hold script name and script selecting button
    wxBoxSizer*  pSeleSizer = new wxBoxSizer( wxHORIZONTAL );

    // lay out with sizers
    Add( pStoneBitmap, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 10 );
    Add( pGridSizer,   0, wxALIGN_CENTER_VERTICAL|wxALL,  5  );

    pGridSizer->Add( m_pBtnComputer, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 3 );
    pGridSizer->Add( m_pSpinLevel,   0, wxALIGN_CENTER_VERTICAL|wxLEFT, 3 );

    pGridSizer->Add( m_pBtnHuman, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 3 );
    pGridSizer->Add( m_pTxtHuman, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 3 );

    pGridSizer->Add( m_pBtnLuaAI, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 3 );
    pGridSizer->Add( pSeleSizer,  0, wxALIGN_CENTER_VERTICAL|wxALL,  0 );

    pSeleSizer->Add( m_pTxtLuaScript, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 3 );
    pSeleSizer->Add( m_pBtnLuaScript, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 3 );
}

////////////////////////////////////////////////////////////////
void wzOptionBox::Init( const wzUInt8& PlayerCode, const std::string& Name )
{
    m_PlayerCode    = PlayerCode;
    wxString wxName = Name.c_str();

    if( (m_PlayerCode & wzCOMPUTER_PLAYER) == wzCOMPUTER_PLAYER )
        InitComputer();
    else if( m_PlayerCode == wzHUMAN_PLAYER )
        InitHuman( wxName );
    else if( m_PlayerCode == wzLUA_AI_PLAYER )
        InitLuaAI( wxName );
}

////////////////////////////////////////////////////////////////
void wzOptionBox::GetOptions( wzUInt8& PlayerCode, std::string& PlayerName )
{
    PlayerCode = m_PlayerCode;

    if( (m_PlayerCode & wzCOMPUTER_PLAYER) == wzCOMPUTER_PLAYER )
        PlayerName = wzOptions::ComputerPlayerName( m_PlayerCode ^ wzCOMPUTER_PLAYER );
    else if( m_PlayerCode == wzHUMAN_PLAYER )
        PlayerName = m_pTxtHuman->GetValue().c_str();
    else if( m_PlayerCode == wzLUA_AI_PLAYER )
        PlayerName = m_pTxtLuaScript->GetValue().c_str();
}

/////////////////////////////////////////
// initialize ctrls related to built-in AI
////////////////////////////////////////////////////////////////
void wzOptionBox::InitComputer()
{
    m_pBtnComputer->SetValue( true );
    m_pSpinLevel->SetValue( static_cast<int>(m_PlayerCode ^ wzCOMPUTER_PLAYER) );
    m_pSpinLevel->Enable( true );

    DisableHuman();
    DisableLuaAI();
}

////////////////////////////////////////
// initialize ctrls related to human player
////////////////////////////////////////////////////////////////
void wzOptionBox::InitHuman( const wxString& Name )
{
    m_pBtnHuman->SetValue( true );
    m_pTxtHuman->SetValue( Name );
    m_pTxtHuman->Enable( true );

    DisableComputer();
    DisableLuaAI();
}

////////////////////////////////////////////
// initialize ctrls related to Lua script player
/////////////////////////////////////////////////////////////
void wzOptionBox::InitLuaAI( const wxString& LuaScript )
{
    m_pBtnLuaAI->SetValue( true );
    m_pTxtLuaScript->SetValue( LuaScript );
    m_pTxtLuaScript->Enable( true );
    m_pBtnLuaScript->Enable( true );

    DisableComputer();
    DisableHuman();
}

/////////////////////////////////////////
// disable ctrls related to built-in AI
/////////////////////////////////////////////////////////////
void wzOptionBox::DisableComputer()
{
    m_pSpinLevel->Enable( false );
    m_pSpinLevel->SetValue( static_cast<int>(wzCOMPUTER_LEVEL1) );
}

////////////////////////////////////////
// disable ctrls related to human player
////////////////////////////////////////////////////////////////
void wzOptionBox::DisableHuman()
{
    m_pTxtHuman->Clear();
    m_pTxtHuman->Enable( false );
}

////////////////////////////////////////////
// disable ctrls related to Lua script player
/////////////////////////////////////////////////////////////
void wzOptionBox::DisableLuaAI()
{
    m_pTxtLuaScript->Clear();
    m_pTxtLuaScript->Enable( false );
    m_pBtnLuaScript->Enable( false );
}

////////////////////////////////////////////////////////////////
void wzOptionBox::OnRadioBtn()
{
    if( m_pBtnComputer->GetValue() )
    {
        m_PlayerCode = wzCOMPUTER_PLAYER + 
                       static_cast<wzUInt8>( m_pSpinLevel->GetValue() );
        m_pSpinLevel->Enable( true );

        DisableHuman();
        DisableLuaAI();
    }
    else if( m_pBtnHuman->GetValue() )
    {
        m_PlayerCode = wzHUMAN_PLAYER;
 
        m_pTxtHuman->SetValue( _(wzHUMAN_PLAYER_NAME) );
        m_pTxtHuman->Enable( true );

        DisableComputer();
        DisableLuaAI();
    }
    else if( m_pBtnLuaAI->GetValue() )
    {
        m_PlayerCode = wzLUA_AI_PLAYER;

        m_pTxtLuaScript->Enable( true );
        m_pBtnLuaScript->Enable( true );

        DisableComputer();
        DisableHuman();
    }
}

////////////////////////////////////////////////////////////////
void wzOptionBox::OnSpinCtrl()
{
    m_PlayerCode = wzCOMPUTER_PLAYER + 
                   static_cast<wzUInt8>( m_pSpinLevel->GetValue() );
}

////////////////////////////////////////////////////////////////
void wzOptionBox::OnSeleBtn()
{
    static const wxString Wildcard = wxString( 
           _("Tick5 script files (*.wz;*.lua)|*.wz;*.lua|All files|*") );

    wxFileDialog dlg( m_Parent, wzAppName + _(" - Choose a script file"), 
                      wxEmptyString, wxEmptyString, Wildcard, 
                      wxOPEN|wxFILE_MUST_EXIST|wxCHANGE_DIR );

    if( dlg.ShowModal() == wxID_OK )
    {
        m_pTxtLuaScript->SetValue( dlg.GetPath() );
    }
}

