// wzOptionsDlg.h - Game options dialog for wx GUI app
// Copyright (C) 2006-2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __WZ_OPTIONS_DLG_H__
#define __WZ_OPTIONS_DLG_H__

#include <string>

class wxSpinCtrl;
class wxSpinEvent;
class wzOptionBox;
class wzOptions;


////////////////////
// Game options dialog
/////////////////////////////////////////////////
class wzOptionsDlg : public wxDialog
{
public:
    wzOptionsDlg( wxWindow* parent, wzOptions& options );
    virtual ~wzOptionsDlg() {}

    void OnBlackRadioBtn( wxCommandEvent& ); 
    void OnWhiteRadioBtn( wxCommandEvent& ); 
    void OnBlackSpinCtrl( wxSpinEvent& ); 
    void OnWhiteSpinCtrl( wxSpinEvent& ); 
    void OnBlackSeleBtn( wxCommandEvent& ); 
    void OnWhiteSeleBtn( wxCommandEvent& ); 

    void GetOptions( wzOptions& options ) const;

private:
    void CreateDlg();
    void CreateBlackBox();
    void CreateWhiteBox();
    void Init( wzOptions& options );

    wzOptionBox*  m_pBlackBox;
    wzOptionBox*  m_pWhiteBox;

    DECLARE_EVENT_TABLE()
};

typedef struct
{
    wxWindowID  IDComputer;
    wxWindowID  IDLevel;
    wxWindowID  IDHuman;
    wxWindowID  IDLuaAI;
    wxWindowID  IDLuaBtn;
    char**      xpm;
} wzBoxParameters;



class wzOptionBox : public wxStaticBoxSizer
{
public:
    wzOptionBox( wxWindow* parent, wzBoxParameters& parameters );
    virtual ~wzOptionBox() {}

    void Init( const wzUInt8& PlayerCode, const std::string& Name );
    void GetOptions( wzUInt8& PlayerCode, std::string& Name );

    void OnRadioBtn(); 
    void OnSpinCtrl(); 
    void OnSeleBtn(); 

private:
    void CreateBox( wzBoxParameters& parameters );
    void InitComputer();
    void InitHuman( const wxString& Name );
    void InitLuaAI( const wxString& LuaScript );
    void DisableComputer();
    void DisableHuman();
    void DisableLuaAI();

private:
    wzUInt8         m_PlayerCode;

    // built-in AI
    wxRadioButton*  m_pBtnComputer;
    wxSpinCtrl*     m_pSpinLevel;

    // human player
    wxRadioButton*  m_pBtnHuman;
    wxTextCtrl*     m_pTxtHuman;

    // Lua script
    wxRadioButton*  m_pBtnLuaAI;
    wxTextCtrl*     m_pTxtLuaScript;
    wxButton*       m_pBtnLuaScript;

    wxWindow*       m_Parent;
};


inline void wzOptionsDlg::OnBlackRadioBtn( wxCommandEvent& e )
{
    m_pBlackBox->OnRadioBtn();
}

inline void wzOptionsDlg::OnWhiteRadioBtn( wxCommandEvent& e )
{
    m_pWhiteBox->OnRadioBtn();
}

inline void wzOptionsDlg::OnBlackSpinCtrl( wxSpinEvent& e )
{
    m_pBlackBox->OnSpinCtrl();
}

inline void wzOptionsDlg::OnWhiteSpinCtrl( wxSpinEvent& e )
{
    m_pWhiteBox->OnSpinCtrl();
}

inline void wzOptionsDlg::OnBlackSeleBtn( wxCommandEvent& e )
{
    m_pBlackBox->OnSeleBtn();
}

inline void wzOptionsDlg::OnWhiteSeleBtn( wxCommandEvent& e )
{
    m_pWhiteBox->OnSeleBtn();
}

#endif
