// wzTickSound.cpp - Tick sound.
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wx/sound.h>

#include "wzTickSound.h"
#include "res/TickSound.h"


/////////////////////////
wzTickSound::wzTickSound()
  : m_pSound( new wxSound() )
{
    m_Loaded  = m_pSound->Create( sizeof(TickSound), TickSound );
    m_Enabled = m_Loaded;
    gettimeofday( &m_Previous, NULL );
}

//////////////////////////
wzTickSound::~wzTickSound()
{
    delete m_pSound;
}

////////////////////////////////////
void wzTickSound::Set( const wzMove& )
{
    if( m_Enabled && IsOK() )  m_pSound->Play();
}

// length of the tick sound is 0.02 sec.
#define MIN_INTERVAL  0.05

////////////////////////////////////
// Return true, if time interval between now and previous play
// is long enough. 
//
// Since wxSound::IsPlaying() is not implemented (as of 2.8.x),
// on wxMSW, this is a work-around to avoid buzzing on some
// occasions when wxSound::Play() is called intensively.
///////////////////////////////////////////////////////////////
bool wzTickSound::IsOK()
{
    timeval  current;
    gettimeofday( &current, NULL );
    double interval = (current.tv_sec - m_Previous.tv_sec) +
           static_cast<double>(current.tv_usec - m_Previous.tv_usec)/1000000.0;

    if( interval > MIN_INTERVAL )
    {
        m_Previous.tv_sec  = current.tv_sec;
        m_Previous.tv_usec = current.tv_usec;

        return true;
    }
    else
    {
        return false;
    }
}
