// wzAIBoardTest.cpp - Unit test for wzAIBoard
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzAIBoardTest.h"
#include <wzAIBoard.h>

#include <TestSuite.h>
#include <TestCaller.h>


void wzAIBoardTest::testCtor()
{
    wzBoardBase bb;
    bb.Set( 7, 7, wzBLACK );
    bb.Set( 2, 10, wzWHITE );

    wzAIBoard ab( bb );
    wzBound box = ab.GetBound();
    assert( box.xMin == 2 );
    assert( box.yMin == 7 );
    assert( box.xMax == 7 );
    assert( box.yMax == 10 );

    ab.Set( 3, 3, wzBLACK );
    box = ab.GetBound();
    assert( box.xMin == 2 );
    assert( box.yMin == 3 );
    assert( box.xMax == 7 );
    assert( box.yMax == 10 );
}

void wzAIBoardTest::testGetBound()
{
    wzBound     box;
    wzBoardBase bb;
    wzAIBoard   b(bb);

    // meaningless, but harmless either
    box = b.GetBound();

    b.Set( 7, 7, wzBLACK );
    box = b.GetBound();
    assert( box.xMin == 7 );
    assert( box.yMin == 7 );
    assert( box.xMax == 7 );
    assert( box.yMax == 7 );

    b.Set( 2, 10, wzWHITE );
    box = b.GetBound();
    assert( box.xMin == 2 );
    assert( box.yMin == 7 );
    assert( box.xMax == 7 );
    assert( box.yMax == 10 );

    b.Set( 3, 3, wzBLACK );
    box = b.GetBound();
    assert( box.xMin == 2 );
    assert( box.yMin == 3 );
    assert( box.xMax == 7 );
    assert( box.yMax == 10 );

    b.Set( 6, 6, wzWHITE );
    box = b.GetBound();
    assert( box.xMin == 2 );
    assert( box.yMin == 3 );
    assert( box.xMax == 7 );
    assert( box.yMax == 10 );

    b.Reset( 6, 6 );
    box = b.GetBound();
    assert( box.xMin == 2 );
    assert( box.yMin == 3 );
    assert( box.xMax == 7 );
    assert( box.yMax == 10 );

    b.Reset( 3, 3 );
    box = b.GetBound();
    assert( box.xMin == 2 );
    assert( box.yMin == 7 );
    assert( box.xMax == 7 );
    assert( box.yMax == 10 );

    b.Reset( 2, 10 );
    box = b.GetBound();
    assert( box.xMin == 7 );
    assert( box.yMin == 7 );
    assert( box.xMax == 7 );
    assert( box.yMax == 7 );

    b.Reset( 2, 10 );

    // meaningless, but harmless either
    box = b.GetBound();

    b.Set( 7, 7, wzBLACK );

    b.Clean();

    b.Set( 0, 0, wzBLACK );
    b.Set( 2, 0, wzBLACK );
    b.Set( 4, 0, wzBLACK );
    b.Set( 6, 0, wzBLACK );
    b.Set( 8, 0, wzBLACK );

    box = b.GetBound();
    assert( box.xMin == 0 );
    assert( box.yMin == 0 );
    assert( box.xMax == 8 );
    assert( box.yMax == 0 );
}

Test* wzAIBoardTest::suite()
{
    TestSuite *suite = new TestSuite( "wzAIBoardTest" );

    suite->addTest( CU_SUIT_TEST( wzAIBoardTest, testCtor ) );
    suite->addTest( CU_SUIT_TEST( wzAIBoardTest, testGetBound ) );

    return suite;
}
