// wzBitBoardTest.cpp - Unit test for wzBitBoard
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzBitBoardTest.h"
#include <wzBitBoard.h>

#include <TestSuite.h>
#include <TestCaller.h>


void wzBitBoardTest::testCtors()
{
    // default
    wzBitBoard bb;
    bb.Set( 1, 2 );
    bb.Set( 2, 2 );

    // copy
    wzBitBoard bb2 = bb;
    assert( bb2 == bb );

    // assignment
    wzBitBoard bb3;
    assert( bb3 != bb );

    bb3 = bb;
    assert( bb3 == bb );
}

void wzBitBoardTest::testOperators()
{
    wzBitBoard bb, bb2, bb3;

    // == 
    assert( bb == bb );
    assert( bb == bb2 );
    assert( bb == bb3 );

    bb.Set ( 1, 2 );
    bb2.Set( 2, 1 );

    // assignment
    bb3 = bb;

    // != 
    assert( bb  == bb3 );
    assert( bb  != bb2 );
    assert( bb3 != bb2 );
    assert( bb2 != bb );
    assert( bb2 != bb3 );

    // &
    assert( (bb3 & bb)  == 1 );
    assert( (bb  & bb3) == 1 );
    assert( (bb2 & bb)  == 0 );
    assert( (bb  & bb2) == 0 );

    bb.Set( 2, 1 );

    assert( (bb3 & bb)  == 1 );
    assert( (bb  & bb3) == 1 );
    assert( (bb2 & bb)  == 1 );
    assert( (bb  & bb2) == 1 );

    // +=
    assert( bb.IsEmpty( 3, 3 ) );
    bb2.Set( 3, 3 );
    bb += bb2;
    assert( !bb.IsEmpty( 3, 3 ) );

    assert( bb.IsEmpty( 3, 2 ) );
    bb3.Set( 3, 2 );
    bb += bb3;
    assert( !bb.IsEmpty( 3, 2 ) );
}

void wzBitBoardTest::testSet()
{
    wzBitBoard bb;

    assert( bb.IsEmpty( 0, 0 ) );
    assert( bb.IsEmpty( 0, 1 ) );
    assert( bb.IsEmpty( 0, 2 ) );
    assert( bb.IsEmpty( 1, 0 ) );
    assert( bb.IsEmpty( 1, 1 ) );
    assert( bb.IsEmpty( 1, 2 ) );
    assert( bb.IsEmpty( 2, 0 ) );
    assert( bb.IsEmpty( 2, 1 ) );
    assert( bb.IsEmpty( 2, 2 ) );

    assert( bb.Set( 0, 0 ) );
    assert( !bb.Set( 0, 0 ) );
    assert( !bb.IsEmpty( 0, 0 ) );
    assert( bb.IsEmpty( 0, 1 ) );
    assert( bb.IsEmpty( 0, 2 ) );
    assert( bb.IsEmpty( 1, 0 ) );
    assert( bb.IsEmpty( 1, 1 ) );
    assert( bb.IsEmpty( 1, 2 ) );
    assert( bb.IsEmpty( 2, 0 ) );
    assert( bb.IsEmpty( 2, 1 ) );
    assert( bb.IsEmpty( 2, 2 ) );

    assert( bb.Set( 0, 1 ) );
    assert( !bb.Set( 0, 1 ) );
    assert( !bb.IsEmpty( 0, 1 ) );
    assert( !bb.IsEmpty( 0, 0 ) );
    assert( bb.IsEmpty( 0, 2 ) );
    assert( bb.IsEmpty( 1, 0 ) );
    assert( bb.IsEmpty( 1, 1 ) );
    assert( bb.IsEmpty( 1, 2 ) );
    assert( bb.IsEmpty( 2, 0 ) );
    assert( bb.IsEmpty( 2, 1 ) );
    assert( bb.IsEmpty( 2, 2 ) );

    assert( bb.Set( 1, 1 ) );
    assert( !bb.Set( 1, 1 ) );
    assert( !bb.IsEmpty( 1, 1 ) );
    assert( !bb.IsEmpty( 0, 0 ) );
    assert( !bb.IsEmpty( 0, 1 ) );
    assert( bb.IsEmpty( 0, 2 ) );
    assert( bb.IsEmpty( 1, 0 ) );
    assert( bb.IsEmpty( 1, 2 ) );
    assert( bb.IsEmpty( 2, 0 ) );
    assert( bb.IsEmpty( 2, 1 ) );
    assert( bb.IsEmpty( 2, 2 ) );

    assert( bb.Set( 2, 2 ) );
    assert( !bb.Set( 2, 2 ) );
    assert( !bb.IsEmpty( 2, 2 ) );
    assert( bb.IsEmpty( 0, 2 ) );
    assert( !bb.IsEmpty( 0, 0 ) );
    assert( !bb.IsEmpty( 0, 1 ) );
    assert( bb.IsEmpty( 1, 0 ) );
    assert( !bb.IsEmpty( 1, 1 ) );
    assert( bb.IsEmpty( 1, 2 ) );
    assert( bb.IsEmpty( 2, 0 ) );
    assert( bb.IsEmpty( 2, 1 ) );

    for( int x = 0; x < wzDimension; ++x )
        for( int y = 0; y < wzDimension; ++y )
            bb.Reset( x, y );

    assert( bb.IsEmpty( 0, 0 ) );
    assert( bb.IsEmpty( 0, 1 ) );
    assert( bb.IsEmpty( 0, 2 ) );
    assert( bb.IsEmpty( 1, 0 ) );
    assert( bb.IsEmpty( 1, 1 ) );
    assert( bb.IsEmpty( 1, 2 ) );
    assert( bb.IsEmpty( 2, 0 ) );
    assert( bb.IsEmpty( 2, 1 ) );
    assert( bb.IsEmpty( 2, 2 ) );
}

void wzBitBoardTest::testReset()
{
    wzBitBoard bb;
    for( int x = 0; x < wzDimension; ++x )
        for( int y = 0; y < wzDimension; ++y )
            bb.Set( x, y );

    bb.Reset( 0, 0 );
    assert( bb.IsEmpty( 0, 0 ) );
    assert( !bb.IsEmpty( 0, 1 ) );
    assert( !bb.IsEmpty( 0, 2 ) );
    assert( !bb.IsEmpty( 1, 0 ) );
    assert( !bb.IsEmpty( 1, 1 ) );
    assert( !bb.IsEmpty( 1, 2 ) );
    assert( !bb.IsEmpty( 2, 0 ) );
    assert( !bb.IsEmpty( 2, 1 ) );
    assert( !bb.IsEmpty( 2, 2 ) );

    bb.Reset( 2, 2 );
    assert( bb.IsEmpty( 0, 0 ) );
    assert( !bb.IsEmpty( 0, 1 ) );
    assert( !bb.IsEmpty( 0, 2 ) );
    assert( !bb.IsEmpty( 1, 0 ) );
    assert( !bb.IsEmpty( 1, 1 ) );
    assert( !bb.IsEmpty( 1, 2 ) );
    assert( !bb.IsEmpty( 2, 0 ) );
    assert( !bb.IsEmpty( 2, 1 ) );
    assert( bb.IsEmpty( 2, 2 ) );
}

void wzBitBoardTest::testClean()
{
    wzBitBoard bb;
    for( int x = 0; x < wzDimension; ++x )
        for( int y = 0; y < wzDimension; ++y )
            assert( bb.IsEmpty( x, y ) );

    bb.Set( 1, 1 );
    assert( !bb.IsEmpty( 1, 1 ) );

    bb.Set( 7, 7 );
    assert( !bb.IsEmpty( 1, 1 ) );

    bb.Clean();
    assert( bb.IsEmpty( 1, 1 ) );
    assert( bb.IsEmpty( 7, 7 ) );
}

Test* wzBitBoardTest::suite()
{
    TestSuite *suite = new TestSuite( "wzBitBoardTest" );

    suite->addTest( CU_SUIT_TEST( wzBitBoardTest, testCtors ) );
    suite->addTest( CU_SUIT_TEST( wzBitBoardTest, testOperators ) );
    suite->addTest( CU_SUIT_TEST( wzBitBoardTest, testSet ) );
    suite->addTest( CU_SUIT_TEST( wzBitBoardTest, testReset ) );
    suite->addTest( CU_SUIT_TEST( wzBitBoardTest, testClean ) );

    return suite;
}
