// wzBoardBaseTest.cpp - Unit test for wzBoardBase
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzBoardBaseTest.h"
#include <wzBoardBase.h>

#include <TestSuite.h>
#include <TestCaller.h>


void wzBoardBaseTest::testIsFullIsEmpty()
{
    wzBoardBase b;
    assert( !b.IsFull() );
    assert(  b.IsEmpty() );


    for( int x = 0; x < wzDimension; ++x )
        for( int y = 0; y < wzDimension; ++y )
            b.Set( x, y, wzBLACK );

    assert(  b.IsFull() );
    assert( !b.IsEmpty() );
}

void wzBoardBaseTest::testPlies()
{
    wzBoardBase b, b2;
    assert( b.Plies() == 0 );

    b.Set( 1, 1, wzBLACK );
    assert( b.Plies() == 1 );
    b.Set( 1, 2, wzBLACK );
    assert( b.Plies() == 2 );

    b2 = b;
    assert( b2.Plies() == 2 );

    wzBoardBase b3 = b2;
    assert( b3.Plies() == 2 );

    b.Clean();
    assert( b.Plies() == 0 );
}

void wzBoardBaseTest::testSet()
{
    wzBoardBase b;
    assert( b.IsEmpty() );

    for( int x = 0; x < wzDimension; ++x )
        for( int y = 0; y < wzDimension-1; ++y )
            b.Set( x, y, wzBLACK );

    assert( !b.IsFull() );

    for( int x = 0; x < wzDimension; ++x )
        b.Set( x, wzDimension-1, wzBLACK );

    assert( b.IsFull() );

    // already taken
    assert( !b.Set( 0, 0, wzBLACK ) );
    assert( !b.Set( 1, 1, wzBLACK ) );
    assert( !b.Set( 2, 2, wzBLACK ) );
} 

void wzBoardBaseTest::testClean()
{
    wzBoardBase b;
    assert( b.IsEmpty() );

    int x, y;
    for( x = 0; x < wzDimension; ++x )
        for( y = 0; y < wzDimension; ++y )
            b.Set( x, y, wzBLACK );

    assert( b.IsFull() );

    for( x = 0; x < wzDimension; ++x )
        for( y = 0; y < wzDimension; ++y )
            b.Reset( x, y );

    assert( b.IsEmpty() );
    assert( !b.IsFull() );
} 

void wzBoardBaseTest::testReset()
{
    wzBoardBase b;
    assert( b.IsEmpty() );
    assert( !b.IsFull() );
    assert( b.Plies() == 0 );
    for( int x = 0; x < wzDimension; ++x )
        for( int y = 0; y < wzDimension; ++y )
            assert( b.IsEmpty( x, y ) );

    b.Set( 1, 1, wzBLACK );
    assert( !b.IsEmpty( 1, 1 ) );
    assert( !b.IsEmpty() );
    assert( !b.IsFull() );
    assert( b.Plies() == 1 );

    b.Set( 7, 7, wzBLACK );
    assert( !b.IsEmpty( 1, 1 ) );

    b.Clean();
    assert( b.IsEmpty( 1, 1 ) );
    assert( b.IsEmpty( 7, 7 ) );
    assert( b.IsEmpty() );
    assert( !b.IsFull() );
    assert( b.Plies() == 0 );
}

void wzBoardBaseTest::testGetStone()
{
    wzBoardBase b;

    assert( b.GetStone( 0, 0 ) == wzNONE );
    b.Set( 0, 0, wzBLACK );
    assert( b.GetStone( 0, 0 ) == wzBLACK );

    assert( b.GetStone( 1, 1 ) == wzNONE );
    b.Set( 1, 1, wzWHITE );
    assert( b.GetStone( 1, 1 ) == wzWHITE );

    assert( b.GetStone( 2, 2 ) == wzNONE );
    b.Set( 2, 2, wzBLACK );
    assert( b.GetStone( 2, 2 ) == wzBLACK );


}

Test* wzBoardBaseTest::suite()
{
    TestSuite *suite = new TestSuite( "wzBoardBaseTest" );

    suite->addTest( CU_SUIT_TEST( wzBoardBaseTest, testIsFullIsEmpty ) );
    suite->addTest( CU_SUIT_TEST( wzBoardBaseTest, testPlies ) );
    suite->addTest( CU_SUIT_TEST( wzBoardBaseTest, testSet ) );
    suite->addTest( CU_SUIT_TEST( wzBoardBaseTest, testReset ) );
    suite->addTest( CU_SUIT_TEST( wzBoardBaseTest, testClean ) );
    suite->addTest( CU_SUIT_TEST( wzBoardBaseTest, testGetStone ) );

    return suite;
}
