// wzBoardTest.cpp - Unit test for wzBoard
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "wzBoardTest.h"
#include <wzBoard.h>

#include <TestSuite.h>
#include <TestCaller.h>


void wzBoardTest::testUndoMove()
{
    wzBoard b;

    // meaningless, but harmless either
    b.UndoMove();

    b.Set( 7, 7, wzBLACK );
    b.Set( 8, 8, wzWHITE );
    b.Set( 7, 8, wzBLACK );
    b.Set( 8, 7, wzWHITE );
    assert( b.GetStone( 7, 7 ) == wzBLACK );
    assert( b.GetStone( 8, 8 ) == wzWHITE );
    assert( b.GetStone( 7, 8 ) == wzBLACK );
    assert( b.GetStone( 8, 7 ) == wzWHITE );

    b.UndoMove();
    assert( b.GetStone( 7, 7 ) == wzBLACK );
    assert( b.GetStone( 8, 8 ) == wzWHITE );
    assert( b.GetStone( 7, 8 ) == wzBLACK );
    assert( b.GetStone( 8, 7 ) == wzNONE );

    b.UndoMove();
    assert( b.GetStone( 7, 7 ) == wzBLACK );
    assert( b.GetStone( 8, 8 ) == wzWHITE );
    assert( b.GetStone( 7, 8 ) == wzNONE );
    assert( b.GetStone( 8, 7 ) == wzNONE );

    b.UndoMove();
    assert( b.GetStone( 7, 7 ) == wzBLACK );
    assert( b.GetStone( 8, 8 ) == wzNONE );
    assert( b.GetStone( 7, 8 ) == wzNONE );
    assert( b.GetStone( 8, 7 ) == wzNONE );

    b.UndoMove();
    assert( b.GetStone( 7, 7 ) == wzNONE );
    assert( b.GetStone( 8, 8 ) == wzNONE );
    assert( b.GetStone( 7, 8 ) == wzNONE );
    assert( b.GetStone( 8, 7 ) == wzNONE );

    // meaningless, but harmless either
    b.UndoMove();

    b.Set( 7, 7, wzBLACK );
    assert( b.GetStone( 7, 7 ) == wzBLACK );
}

void wzBoardTest::testGetLastMove()
{
    wzMove  m;
    wzBoard b;

    // meaningless, but harmless either
    m = b.GetLastMove();

    b.Set( 7, 7, wzBLACK );
    m = b.GetLastMove();
    assert( m.X == 7 );
    assert( m.Y == 7 );
    assert( m.Stone == wzBLACK );

    b.Set( 2, 10, wzWHITE );
    m = b.GetLastMove();
    assert( m.X == 2 );
    assert( m.Y == 10 );
    assert( m.Stone == wzWHITE );

    b.Set( 3, 3, wzBLACK );
    m = b.GetLastMove();
    assert( m.X == 3 );
    assert( m.Y == 3 );
    assert( m.Stone == wzBLACK );

    b.Set( 6, 6, wzWHITE );
    m = b.GetLastMove();
    assert( m.X == 6 );
    assert( m.Y == 6 );
    assert( m.Stone == wzWHITE );

    b.Reset( 6, 6 );
    m = b.GetLastMove();
    assert( m.X == 3 );
    assert( m.Y == 3 );
    assert( m.Stone == wzBLACK );

    b.Reset( 3, 3 );
    m = b.GetLastMove();
    assert( m.X == 2 );
    assert( m.Y == 10 );
    assert( m.Stone == wzWHITE );

    b.Reset( 2, 10 );

    // meaningless, but harmless either
    m = b.GetLastMove();

    // try again to make sure it won't crash test
    b.Reset( 2, 10 );
    m = b.GetLastMove();

    b.Set( 7, 7, wzBLACK );

    b.Clean();

    b.Set( 0, 0, wzBLACK );
    b.Set( 2, 0, wzBLACK );
    b.Set( 4, 0, wzBLACK );
    b.Set( 6, 0, wzBLACK );
    b.Set( 8, 0, wzBLACK );

    m = b.GetLastMove();
    assert( m.X == 8 );
    assert( m.Y == 0 );
    assert( m.Stone == wzBLACK );
}

Test* wzBoardTest::suite()
{
    TestSuite *suite = new TestSuite( "wzBoardTest" );

    suite->addTest( CU_SUIT_TEST( wzBoardTest, testUndoMove ) );
    suite->addTest( CU_SUIT_TEST( wzBoardTest, testGetLastMove ) );

    return suite;
}
