// wzFieldObserverTest.cpp - Unit test for wzFieldObserver
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <TestSuite.h>
#include <TestCaller.h>

#include "wzFieldObserverTest.h"
#include <wzFieldObserver.h>
#include <wzBoardBase.h>


void wzFieldObserverTest::testFieldObserver()
{
    wzBoardBase b;
    wzFieldObserver ob;
    short x, y;

    // meaningless, but harmless call
    const wzBitBoard& field = ob.GetFields();

    b.Set( 7, 7, wzBLACK );

    // test wzFieldObserver::Init()
    b.Register( &ob );
    //field = ob.GetFields();

    // horizontal
    for( x = 5, y = 7; x <= 9; x++ )
        assert( !field.IsEmpty( x, y ) );

    // vertical
    for( x = 7, y = 5; y <= 9; y++ )
        assert( !field.IsEmpty( x, y ) );

    // diagonal forwards
    for( x = 5, y = 5; x <= 9; x++, y++ )
        assert( !field.IsEmpty( x, y ) );

    // diagonal backwards
    for( x = 9, y = 5; y <= 9; x--, y++ )
        assert( !field.IsEmpty( x, y ) );

    // test wzFieldObserver::Set()
    b.Set( 2, 10, wzWHITE );
    assert( !field.IsEmpty( 2, 10 ) );
    assert( !field.IsEmpty( 0, 10 ) );
    assert( !field.IsEmpty( 4, 10 ) );
    assert( !field.IsEmpty( 2, 8 ) );
    assert( !field.IsEmpty( 2, 12 ) );

    // horizontal
    for( x = 0, y = 10; x <= 4; x++ )
        assert( !field.IsEmpty( x, y ) );

    // vertical
    for( x = 2, y = 8; y <= 12; y++ )
        assert( !field.IsEmpty( x, y ) );

    // diagonal forwards
    for( x = 0, y = 8; x <= 4; x++, y++ )
        assert( !field.IsEmpty( x, y ) );

    // diagonal backwards
    assert( !field.IsEmpty( 4, 8 ) );
    assert( !field.IsEmpty( 3, 9 ) );
    assert( !field.IsEmpty( 2, 10 ) );
    assert( !field.IsEmpty( 1, 11 ) );
    assert( !field.IsEmpty( 0, 12 ) );

    b.Set( 3, 3, wzBLACK );
    assert( !field.IsEmpty( 1, 1 ) );
    assert( !field.IsEmpty( 3, 3 ) );
    assert( !field.IsEmpty( 1, 3 ) );
    assert( !field.IsEmpty( 5, 3 ) );
    assert( !field.IsEmpty( 3, 1 ) );
    assert( !field.IsEmpty( 3, 5 ) );

    b.Set( 6, 6, wzWHITE );
    assert( !field.IsEmpty( 8, 4 ) );

    // test wzFieldObserver::Reset( m )
    b.Reset( 6, 6 );
    assert( field.IsEmpty( 8, 4 ) );

    b.Reset( 3, 3 );
    assert( field.IsEmpty( 1, 1 ) );
    assert( field.IsEmpty( 5, 1 ) );

    // GetFields(), get back to initial field
    b.Reset( 2, 10 );
    // horizontal
    for( x = 5, y = 7; x <= 9; x++ )
        assert( !field.IsEmpty( x, y ) );

    // vertical
    for( x = 7, y = 5; y <= 9; y++ )
        assert( !field.IsEmpty( x, y ) );

    // diagonal forwards
    for( x = 5, y = 5; x <= 9; x++, y++ )
        assert( !field.IsEmpty( x, y ) );

    // diagonal backwards
    for( x = 9, y = 5; y <= 9; x--, y++ )
        assert( !field.IsEmpty( x, y ) );

    // make sure it won't crash test, 
    // but filed is meaningless
    b.Reset( 2, 10 );

    // test wzFieldObserver::Clean()
    b.Set( 7, 7, wzBLACK );
    b.Clean();
    b.Set( 0, 0, wzBLACK );
    b.Set( 2, 0, wzBLACK );
    b.Set( 4, 0, wzBLACK );
    b.Set( 6, 0, wzBLACK );
    b.Set( 8, 0, wzBLACK );
    // horizontal
    for( x = 5, y = 7; x <= 9; x++ )
        assert( !field.IsEmpty( x, y ) );

    // vertical
    for( x = 7, y = 5; y <= 9; y++ )
        assert( !field.IsEmpty( x, y ) );

    // diagonal forwards
    for( x = 5, y = 5; x <= 9; x++, y++ )
        assert( !field.IsEmpty( x, y ) );

    // diagonal backwards
    for( x = 9, y = 5; y <= 9; x--, y++ )
        assert( !field.IsEmpty( x, y ) );
}

Test* wzFieldObserverTest::suite()
{
    TestSuite *suite = new TestSuite( "wzFieldObserverTest" );

    suite->addTest( CU_SUIT_TEST( wzFieldObserverTest, testFieldObserver ) );

    return suite;
}
