// wzMoveObserverTest.cpp - Unit test for wzMoveObserver
// Copyright (C) 2007  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <TestSuite.h>
#include <TestCaller.h>

#include "wzMoveObserverTest.h"
#include <wzMoveObserver.h>
#include <wzBoardBase.h>


void wzMoveObserverTest::testCtor()
{
    wzMoveObserver ob;

    wzMove m = ob.GetLastMove();
    assert( m.X == 0 );
    assert( m.Y == 0 );
    assert( m.Stone == wzNONE );
}

void wzMoveObserverTest::testSetMoves()
{
    wzMoveObserver ob;

    wzMove m;
    m.X = 7;
    m.Y = 7;
    m.Stone = wzBLACK;
    ob.Set( m );

    wzMoves moves;
    ob.CopyMoves( moves );

    wzMoveObserver ob2;

    ob2.SetMoves( moves );
    wzMove m2 = ob2.GetLastMove();
    assert( m2.X == 7 );
    assert( m2.Y == 7 );
    assert( m2.Stone == wzBLACK );
}

void wzMoveObserverTest::testMoveObserver()
{
    wzMove m;
    wzBoardBase b;
    wzMoveObserver ob;

    b.Set( 7, 7, wzBLACK );

    b.Register( &ob );

    // won't crash test, but m is meaningless
    m = ob.GetLastMove();

    // test wzMoveObserver::Set()
    b.Set( 2, 10, wzWHITE );
    m = ob.GetLastMove();
    assert( m.X == 2 );
    assert( m.Y == 10 );
    assert( m.Stone == wzWHITE );

    b.Set( 3, 3, wzBLACK );
    m = ob.GetLastMove();
    assert( m.X == 3 );
    assert( m.Y == 3 );
    assert( m.Stone == wzBLACK );

    b.Set( 6, 6, wzWHITE );
    m = ob.GetLastMove();
    assert( m.X == 6 );
    assert( m.Y == 6 );
    assert( m.Stone == wzWHITE );

    // test wzMoveObserver::Reset( m )
    b.Reset( 6, 6 );
    m = ob.GetLastMove();
    assert( m.X == 3 );
    assert( m.Y == 3 );
    assert( m.Stone == wzBLACK );

    b.Reset( 3, 3 );
    m = ob.GetLastMove();
    assert( m.X == 2 );
    assert( m.Y == 10 );
    assert( m.Stone == wzWHITE );

    // won't crash test, but m is meaningless, 
    b.Reset( 2, 10 );
    m = ob.GetLastMove();

    // try again to make sure it won't crash test
    b.Reset( 2, 10 );
    m = ob.GetLastMove();

    // test wzMoveObserver::Clean()
    b.Set( 7, 7, wzBLACK );
    b.Clean();

    b.Set( 0, 0, wzBLACK );
    b.Set( 2, 0, wzBLACK );
    b.Set( 4, 0, wzBLACK );
    b.Set( 6, 0, wzBLACK );
    b.Set( 8, 0, wzBLACK );
    m = ob.GetLastMove();
    assert( m.X == 8 );
    assert( m.Y == 0 );
    assert( m.Stone == wzBLACK );
}

Test* wzMoveObserverTest::suite()
{
    TestSuite *suite = new TestSuite( "wzMoveObserverTest" );

    suite->addTest( CU_SUIT_TEST( wzMoveObserverTest, testCtor ) );
    suite->addTest( CU_SUIT_TEST( wzMoveObserverTest, testSetMoves ) );
    suite->addTest( CU_SUIT_TEST( wzMoveObserverTest, testMoveObserver ) );

    return suite;
}
