// wzRefereeTest.cpp - Unit test for wzReferee
// Copyright (C) 2006  XMartin Yao
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <TestSuite.h>
#include <TestCaller.h>

#include "wzRefereeTest.h"
#include <wzReferee.h>
#include <wzBoard.h>

// test Referee with wzBoardBase
void wzRefereeTest::testWinnerBase()
{
    wzMove       m;
    wzBound      row;
    wzReferee    ref;
    wzBoardBase  b;

    // a bug cought by this pattern
    b.Clean();
    b.Set( 5, 5, wzBLACK );
    b.Set( 10,5, wzBLACK );
    b.Set( 11,6, wzBLACK );
    b.Set( 9, 7, wzBLACK );
    b.Set( 10,10, wzBLACK );
    b.Set( 14,13, wzBLACK );
    m.X = 14;
    m.Y = 13;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m ) == wzNONE );

    // horizontal
    b.Clean();
    b.Set( 0, 0, wzBLACK );
    b.Set( 2, 0, wzBLACK );
    b.Set( 4, 0, wzBLACK );
    b.Set( 6, 0, wzBLACK );
    b.Set( 8, 0, wzBLACK );
    m.X = 8;
    m.Y = 0;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m ) == wzNONE );

    b.Clean();
    b.Set( 5, 5, wzBLACK );
    b.Set( 6, 5, wzBLACK );
    b.Set( 7, 5, wzBLACK );
    b.Set( 8, 5, wzBLACK );
    b.Set( 9, 5, wzBLACK );
    m.X = 9;
    m.Y = 5;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m, row ) == wzBLACK );
    assert( row.xMin == 5 );
    assert( row.yMin == 5 );
    assert( row.xMax == 9 );
    assert( row.yMax == 5 );

    b.Clean();
    b.Set( 5, 5, wzWHITE );
    b.Set( 6, 5, wzWHITE );
    b.Set( 7, 5, wzWHITE );
    b.Set( 8, 5, wzWHITE );
    b.Set( 9, 5, wzWHITE );
    m.X = 9;
    m.Y = 5;
    m.Stone = wzWHITE;
    assert( ref.Winner( b, m, row ) == wzWHITE );
    assert( row.xMin == 5 );
    assert( row.yMin == 5 );
    assert( row.xMax == 9 );
    assert( row.yMax == 5 );

    // vertical
    b.Clean();
    b.Set( 5, 3, wzBLACK );
    b.Set( 5, 5, wzBLACK );
    b.Set( 5, 7, wzBLACK );
    b.Set( 5, 9, wzBLACK );
    b.Set( 5, 11, wzBLACK );
    m.X = 5;
    m.Y = 11;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m ) == wzNONE );

    b.Clean();
    b.Set( 5, 5, wzBLACK );
    b.Set( 5, 6, wzBLACK );
    b.Set( 5, 7, wzBLACK );
    b.Set( 5, 8, wzBLACK );
    b.Set( 5, 9, wzBLACK );
    m.X = 5;
    m.Y = 9;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m, row ) == wzBLACK );
    assert( row.xMin == 5 );
    assert( row.yMin == 5 );
    assert( row.xMax == 5 );
    assert( row.yMax == 9 );

    // diagonal
    b.Clean();
    b.Set( 4, 4, wzBLACK );
    b.Set( 6, 6, wzBLACK );
    b.Set( 7, 7, wzBLACK );
    b.Set( 8, 8, wzBLACK );
    b.Set( 9, 9, wzBLACK );
    m.X = 9;
    m.Y = 9;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m ) == wzNONE );

    b.Clean();
    b.Set( 5, 5, wzBLACK );
    b.Set( 6, 6, wzBLACK );
    b.Set( 7, 7, wzBLACK );
    b.Set( 8, 8, wzBLACK );
    b.Set( 9, 9, wzBLACK );
    m.X = 9;
    m.Y = 9;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m, row ) == wzBLACK );
    assert( row.xMin == 5 );
    assert( row.yMin == 5 );
    assert( row.xMax == 9 );
    assert( row.yMax == 9 );

    // diagonal
    b.Clean();
    b.Set( 9, 5, wzBLACK );
    b.Set( 8, 6, wzBLACK );
    b.Set( 7, 7, wzBLACK );
    b.Set( 6, 8, wzBLACK );
    b.Set( 4, 9, wzBLACK );
    m.X = 4;
    m.Y = 9;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m ) == wzNONE );

    b.Clean();
    b.Set( 9, 5, wzBLACK );
    b.Set( 8, 6, wzBLACK );
    b.Set( 7, 7, wzBLACK );
    b.Set( 6, 8, wzBLACK );
    b.Set( 5, 9, wzBLACK );
    m.X = 9;
    m.Y = 5;
    m.Stone = wzBLACK;
    assert( ref.Winner( b, m, row ) == wzBLACK );
    assert( row.xMin == 5 );
    assert( row.yMin == 9 );
    assert( row.xMax == 9 );
    assert( row.yMax == 5 );

    short x, y;

    b.Clean();
    for( x = y = 9; x < 14 && y < 14; ++x, ++y )
        b.Set( x, y, wzWHITE );

    m.X = 13;
    m.Y = 13;
    m.Stone = wzWHITE;
    assert( ref.Winner( b, m, row ) == wzWHITE );
    assert( row.xMin == 9 );
    assert( row.yMin == 9 );
    assert( row.xMax == 13 );
    assert( row.yMax == 13 );

    b.Clean();
    // horizontal
    for( y = 0; y < wzDimension-5; y++ )
    {
        for( x = y; x < y+5; ++x )
            b.Set( x, y, wzBLACK );

        m.X = x-1;
        m.Y = y;
        m.Stone = wzBLACK;
        assert( ref.Winner( b, m ) == wzBLACK );

        for( x = y; x < y+5; ++x )
            b.Reset( x, y );

        //b.Clean();
        assert( ref.Winner( b, m ) == wzNONE );
    }

    // vertical
    for( x = 0; x < wzDimension-5; x++ )
    {
        for( y = x; y < x+5; ++y )
            b.Set( x, y, wzBLACK );

        m.X = x;
        m.Y = y-1;
        m.Stone = wzBLACK;
        assert( ref.Winner( b, m ) == wzBLACK );

        for( y = x; y < x+5; ++y )
            b.Reset( x, y );

        //b.Clean();
        assert( ref.Winner( b, m ) == wzNONE );
    }

    // diagonal forwards
    for( short i = 0; i < wzDimension-5; ++i )
    {
        for( x = y = i; x < i+5 && y < i+5; ++x, ++y )
            b.Set( x, y, wzBLACK );

        m.X = x-1;
        m.Y = y-1;
        m.Stone = wzBLACK;
        assert( ref.Winner( b, m ) == wzBLACK );

        for( x = y = i; x < i+5 && y < i+5; ++x, ++y )
            b.Reset( x, y );

        assert( ref.Winner( b, m ) == wzNONE );
    }

    // diagonal backwards
    for( short j = 0; j < wzDimension-5; ++j )
    {
        for( x = j+4, y = j; x >=j && y < j+5; --x, ++y )
            b.Set( x, y, wzBLACK );

        m.X = x+1;
        m.Y = y-1;
        m.Stone = wzBLACK;
        assert( ref.Winner( b, m ) == wzBLACK );

        for( x = j+4, y = j; x >=j && y < j+5; --x, ++y )
            b.Reset( x, y );

        assert( ref.Winner( b, m ) == wzNONE );
    }
}

void wzRefereeTest::testWinner()
{
    wzReferee  ref;
    wzBoard    b;

    assert( ref.Winner( b ) == wzNONE );

    // a bug cought by this pattern
    b.Clean();
    b.Set( 5, 5, wzBLACK );
    b.Set( 10,5, wzBLACK );
    b.Set( 11,6, wzBLACK );
    b.Set( 9, 7, wzBLACK );
    b.Set( 10,10, wzBLACK );
    b.Set( 14,13, wzBLACK );
    assert( ref.Winner( b ) == wzNONE );

    // horizontal
    b.Clean();
    b.Set( 0, 0, wzBLACK );
    b.Set( 2, 0, wzBLACK );
    b.Set( 4, 0, wzBLACK );
    b.Set( 6, 0, wzBLACK );
    b.Set( 8, 0, wzBLACK );
    assert( ref.Winner( b ) == wzNONE );

    b.Clean();
    b.Set( 5, 5, wzBLACK );
    b.Set( 6, 5, wzBLACK );
    b.Set( 7, 5, wzBLACK );
    b.Set( 8, 5, wzBLACK );
    b.Set( 9, 5, wzBLACK );
    assert( ref.Winner( b ) == wzBLACK );

    b.Clean();
    b.Set( 5, 5, wzWHITE );
    b.Set( 6, 5, wzWHITE );
    b.Set( 7, 5, wzWHITE );
    b.Set( 8, 5, wzWHITE );
    b.Set( 9, 5, wzWHITE );
    assert( ref.Winner( b ) == wzWHITE );

    // vertical
    b.Clean();
    b.Set( 5, 3, wzBLACK );
    b.Set( 5, 5, wzBLACK );
    b.Set( 5, 7, wzBLACK );
    b.Set( 5, 9, wzBLACK );
    b.Set( 5, 11, wzBLACK );
    assert( ref.Winner( b ) == wzNONE );

    b.Clean();
    b.Set( 5, 5, wzBLACK );
    b.Set( 5, 6, wzBLACK );
    b.Set( 5, 7, wzBLACK );
    b.Set( 5, 8, wzBLACK );
    b.Set( 5, 9, wzBLACK );

    // diagonal
    b.Clean();
    b.Set( 4, 4, wzBLACK );
    b.Set( 6, 6, wzBLACK );
    b.Set( 7, 7, wzBLACK );
    b.Set( 8, 8, wzBLACK );
    b.Set( 9, 9, wzBLACK );
    assert( ref.Winner( b ) == wzNONE );

    b.Clean();
    b.Set( 5, 5, wzBLACK );
    b.Set( 6, 6, wzBLACK );
    b.Set( 7, 7, wzBLACK );
    b.Set( 8, 8, wzBLACK );
    b.Set( 9, 9, wzBLACK );
    assert( ref.Winner( b ) == wzBLACK );

    // diagonal
    b.Clean();
    b.Set( 9, 5, wzBLACK );
    b.Set( 8, 6, wzBLACK );
    b.Set( 7, 7, wzBLACK );
    b.Set( 6, 8, wzBLACK );
    b.Set( 4, 9, wzBLACK );
    assert( ref.Winner( b ) == wzNONE );

    b.Clean();
    b.Set( 9, 5, wzBLACK );
    b.Set( 8, 6, wzBLACK );
    b.Set( 7, 7, wzBLACK );
    b.Set( 6, 8, wzBLACK );
    b.Set( 5, 9, wzBLACK );
    assert( ref.Winner( b ) == wzBLACK );

    short x, y;

    b.Clean();
    for( x = y = 9; x < 14 && y < 14; ++x, ++y )
        b.Set( x, y, wzWHITE );

    assert( ref.Winner( b ) == wzWHITE );

    b.Clean();
    // horizontal
    for( y = 0; y < wzDimension-5; y++ )
    {
        for( x = y; x < y+5; ++x )
            b.Set( x, y, wzBLACK );

        assert( ref.Winner( b ) == wzBLACK );

        b.Clean();
        assert( ref.Winner( b ) == wzNONE );
    }

    // vertical
    for( x = 0; x < wzDimension-5; x++ )
    {
        for( y = x; y < x+5; ++y )
            b.Set( x, y, wzBLACK );

        assert( ref.Winner( b ) == wzBLACK );

        b.Clean();
        assert( ref.Winner( b ) == wzNONE );
    }

    // diagonal forwards
    for( short i = 0; i < wzDimension-5; ++i )
    {
        for( x = y = i; x < i+5 && y < i+5; ++x, ++y )
            b.Set( x, y, wzBLACK );

        assert( ref.Winner( b ) == wzBLACK );

        b.Clean();
        assert( ref.Winner( b ) == wzNONE );
    }

    // diagonal backwards
    for( short j = 0; j < wzDimension-5; ++j )
    {
        for( x = j+4, y = j; x >=j && y < j+5; --x, ++y )
            b.Set( x, y, wzBLACK );

        assert( ref.Winner( b ) == wzBLACK );

        b.Clean();
        assert( ref.Winner( b ) == wzNONE );
    }
}

Test* wzRefereeTest::suite()
{
    TestSuite *suite = new TestSuite( "wzRefereeTest" );

    suite->addTest( CU_SUIT_TEST( wzRefereeTest, testWinnerBase ) );
    suite->addTest( CU_SUIT_TEST( wzRefereeTest, testWinner ) );

    return suite;
}
